﻿using Microsoft.AspNetCore.Builder;
using Microsoft.Extensions.DependencyInjection;
using SkyApm.Tracing;

namespace BPA.Component.APMClient
{
    /// <summary>
    /// ApmConfigExtensions
    /// </summary>
    public static class ApmConfigExtensions
    {
        /// <summary>
        /// 启用APM
        /// </summary>
        /// <param name="app"></param>
        public static void UseAPM(this IApplicationBuilder app)
        {
            //写入apm 的跟踪Id
            app.Use(async (context, next) =>
            {
                var tracking = context.RequestServices.GetService<IEntrySegmentContextAccessor>();

                if (tracking is {Context.TraceId: { }})
                {
                    context.TraceIdentifier = tracking.Context.TraceId;
                }

                await next.Invoke();
            });
        }

        /// <summary>
        /// 获取APM 当前上下文
        /// </summary>
        /// <param name="serviceProvider"></param>
        /// <returns></returns>
        public static IEntrySegmentContextAccessor? GetAPMContextAccessor(this IServiceProvider serviceProvider)
        {
            return serviceProvider.GetService<IEntrySegmentContextAccessor>();
        }
    }
}