﻿using Com.Ctrip.Framework.Apollo;
using Com.Ctrip.Framework.Apollo.Enums;
using Microsoft.Extensions.Configuration;

namespace BPA.Component.ApolloClient
{
    /// <summary>
    /// BPA系统配置基类
    /// </summary>
    // ReSharper disable once InconsistentNaming
    public abstract class ApolloBPAConfig<TApolloConfigModel> : ApolloBaseConfig
        where TApolloConfigModel : ApolloBPAConfig<TApolloConfigModel>
    {
        /// <summary>
        /// OnConfigChange
        /// </summary>
        public event OnConfigChangeEvent<TApolloConfigModel> OnConfigChange;

        /// <summary>
        /// 是否调试
        /// </summary>
        [AutoWrite]
        public bool IsDebug { set; get; } = true;

        #region Common

        /// <summary>
        /// Redis 配置
        /// </summary>
        // [AutoWrite]
        // ReSharper disable once CollectionNeverUpdated.Global
        public List<string> RedisConfig { protected set; get; }

        /// <summary>
        /// RabbitMQ 配置 | 要用就单独在项目new一个新的对象
        /// </summary>
        // [AutoWrite]
        public dynamic RabbitMqConfig { set; get; }

        /// <summary>
        /// Mqtt client 配置 | 要用就单独在项目new一个新的对象
        /// </summary>
        // [AutoWrite]
        public dynamic MqttClientConfig { get; set; }

        /// <summary>
        /// Mqtt server 配置 | 要用就单独在项目new一个新的对象
        /// </summary>
        // [AutoWrite]
        public dynamic MqttServerConfig { get; set; }

        #endregion

        #region MongoDB

        /// <summary>
        /// MongoDB [业务日志] | 要用就单独在项目new一个新的对象
        /// </summary>
        // [AutoWrite]
        public dynamic MongoDbBizLogs { protected set; get; }

        /// <summary>
        /// MongoDB [消息] | 要用就单独在项目new一个新的对象
        /// </summary>
        // [AutoWrite]
        public dynamic MongoDbMessages { protected set; get; }

        /// <summary>
        /// MongoDB [附件] | 要用就单独在项目new一个新的对象
        /// </summary>
        // [AutoWrite]
        public dynamic MongoDbAnnexs { protected set; get; }

        #endregion

        #region MySql

        /// <summary>
        /// MySqlDb | 要用就单独在项目new一个新的对象
        /// </summary>
        // [AutoWrite]
        public dynamic MySqlDb { protected set; get; }

        #endregion


        // /// <summary>
        // ///  ClickHouseDB [统计] 
        // /// </summary>
        // public dynamic ClickHouseDbStatistics { protected set; get; }

        /// <summary>
        /// 构造方法
        /// </summary>
        /// <param name="apolloConfigurationManager"></param>
        /// <param name="configuration"></param>
#pragma warning disable CS0618
#pragma warning disable CS8618
        protected ApolloBPAConfig(ApolloConfigurationManager apolloConfigurationManager, IConfiguration configuration) : base(
#pragma warning restore CS8618
            apolloConfigurationManager, configuration)
#pragma warning restore CS0618
        {
        }

        /// <summary>
        /// 当配置变化时触发,子类可以重写实现自己的逻辑,重写方法里面请务必调用父类 OnApplicationConfigConfigChanged，否则OnConfigChange将不会执行
        /// </summary> 
        /// <param name="key">配置Key</param>
        /// <param name="newValue">配置新值</param>
        /// <param name="oldValue">配置旧值</param>
        /// <param name="changeType">配置变化类型</param> 
        protected override void OnApplicationConfigConfigChanged(string key, string? newValue, string? oldValue,
            PropertyChangeType changeType)
        {
            base.OnApplicationConfigConfigChanged(key, newValue, oldValue, changeType);
            OnConfigChange(this as TApolloConfigModel ?? throw new InvalidOperationException(), key, newValue, oldValue, changeType);
        }
    }

    /// <summary>
    /// 当配置改变的委托
    /// </summary>
    /// <typeparam name="TApolloConfigModel"></typeparam>
    /// <param name="config"></param>
    /// <param name="key"></param>
    /// <param name="newValue"></param>
    /// <param name="oldValue"></param>
    /// <param name="changeType"></param>
    public delegate void OnConfigChangeEvent<in TApolloConfigModel>(TApolloConfigModel config,
        string key,
        string? newValue,
        string? oldValue,
        PropertyChangeType changeType)
        where TApolloConfigModel : ApolloBPAConfig<TApolloConfigModel>;
}