﻿using System.Reflection;
using Newtonsoft.Json;

namespace BPA.Component.ApolloClient
{
    /// <summary>
    /// 自动写配置值
    /// </summary>
    [AttributeUsage(AttributeTargets.Property, AllowMultiple = false, Inherited = false)]
    public class AutoWriteAttribute : Attribute
    {
        /// <summary>
        /// 默认值
        /// </summary>
        public object? DefaultValue { internal set; get; }

        /// <summary>
        /// 默认为当前属性名称,key 为空或者空字符串时，默认值为当前属性名称
        /// </summary> 
        public string Key { set; get; }

        /// <summary>
        /// PropertyInfo
        /// </summary>
        internal PropertyInfo PropertyInfo { set; get; }

        /// <summary>
        /// AutoWriteConfigAttribute  
        /// </summary>  
        public AutoWriteAttribute(object? defaultValue, string key, PropertyInfo propertyInfo)
        {
            DefaultValue = defaultValue;
            Key = key;
            PropertyInfo = propertyInfo;
        }

        /// <summary>
        /// AutoWriteConfigAttribute  
        /// </summary>  
        public AutoWriteAttribute()
        {
        }

        /// <summary>
        /// 设置各类型值
        /// </summary>
        /// <param name="thatConfigObj"></param>
        /// <param name="value"></param>
        internal void SetValue(object thatConfigObj, string? value)
        {
            var valueObj = DefaultValue;
            try
            {
                //如果为空或者空字符串 使用默认值
                if (string.IsNullOrEmpty(value))
                {
                    valueObj = DefaultValue;
                }
                //如果为string 类型 直接等于value
                else if (PropertyInfo.PropertyType == typeof(string))
                {
                    valueObj = value;
                }
                //值类型 使用 JsonConvert 进行转换
                else if (PropertyInfo.PropertyType.IsPrimitive || PropertyInfo.PropertyType == typeof(DateTime))
                {
                    valueObj = JsonConvert.DeserializeObject($"\"{value}\"", PropertyInfo.PropertyType);
                }
                //其他复杂引用类型 使用 JsonConvert 进行转换
                else
                {
                    valueObj = JsonConvert.DeserializeObject(value, PropertyInfo.PropertyType);
                }

                PropertyInfo.SetValue(thatConfigObj, valueObj);
                Console.WriteLine($"*apollo* auto write info: set [{Key}]={value}");
            }
            catch (Exception ex)
            {
                //设置默认值
                PropertyInfo.SetValue(thatConfigObj, valueObj);
                Console.WriteLine(ex);
                Console.WriteLine(
                    $"*apollo* auto write error: key:{Key}," +
                    $"value:{value}," +
                    $"set defaultValue:{JsonConvert.SerializeObject(DefaultValue)}");
            }
        }
    }
}