﻿using BPA.Component.ApolloClient;
using BPA.Component.Extensions;
using BPA.Component.RedisClient;
using CSRedis;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;

namespace BPA.Component.ApolloClientTester
{
    internal class Program
    {
        static void Main(string[] args)
        {
            // Console.WriteLine("1=测试单例,2=测试工厂");
            // var cmd = Console.ReadLine();
            // switch (cmd)
            // {
            //     case "1":
            TestUseRedisSingleton();
            //         break;
            // case "2":
            // TestUseRedisFactory();
            //         break;
            //     default:
            //         Console.WriteLine("命令错误");
            //         break;
            // }

            Console.ReadLine();
        }

        /// <summary>
        /// 测试单例
        /// </summary>
        private static void TestUseRedisSingleton()
        {
            var builder = new ConfigurationBuilder();
            builder.AddEnvironmentVariables();
            IConfiguration configuration = builder.Build();
            configuration.AddApolloConfiguration();
            var services = new ServiceCollection();
            services.AddApollo<TestConfig>();

            services.UseRedisSingleton(sp =>
                GetRedisSingletonConnectionString(sp.GetService<TestConfig>() ?? throw new InvalidOperationException()));

            //构建容器提供对象
            var serviceProvider = services.BuildServiceProvider();
            var redisClient = serviceProvider.GetService<CSRedisClient>();
            var testConfig = serviceProvider.GetService<TestConfig>();

            //扫描
            const string employeeLoginAuthKey = "testfKeys:*";
            if (redisClient != null)
            {
                var scanKeys = redisClient.Keys(employeeLoginAuthKey);
                foreach (var key in scanKeys)
                {
                    redisClient.Del(key);
                }
            }

            const string redisKey = "testfKeys";
            var fKeys = new[] {"1", "2", "3", "4"};
            redisClient?.CacheShell(redisKey, fKeys, -1, keys =>
            {
                var result = new (string, string)[keys.Length];
                for (var i = 0; i < keys.Length; i++)
                {
                    var item = keys[i];
                    result[i] = (item, null);
                }

                // return result;
                return result;
            });

            //测试操作redis
            TestRedisClient(redisClient, "s01");

            //添加配置改变时的事件
            if (testConfig != null)
                testConfig.OnConfigChange += (config, key, newValue, oldValue, changeType) =>
                {
                    if (nameof(config.RedisConfig) != key) return;
                    var conStr = GetRedisSingletonConnectionString(config);
                    //触发生成新的实例
                    RedisConfigExtensions.OnRedisSingletonConnectionStringChange(conStr);
                    redisClient = serviceProvider.GetService<CSRedisClient>();
                    //测试操作redis
                    TestRedisClient(redisClient, "s02");
                };
        }


        // /// <summary>
        // /// 测试redis 工厂
        // /// </summary>
        // static void TestUseRedisFactory()
        // {
        //     var builder = new ConfigurationBuilder();
        //     builder.AddEnvironmentVariables();
        //     var configuration = builder.Build();
        //     configuration.AddApolloConfiguration();
        //
        //     var services = new ServiceCollection();
        //     services.UseApollo<TestConfig>();
        //
        //     //注入单例redis 工厂,
        //     services.UseRedisFactory<TestRedisClientFactory>();
        //
        //     //构建容器提供对象
        //     var serviceProvider = services.BuildServiceProvider();
        //     //从容器中获取redis client 工厂 对象 
        //     var redisClientFactory = serviceProvider.GetService<TestRedisClientFactory>();
        //
        //     //测试操作redis
        //     TestRedisClient(redisClientFactory.CSRedisClient01, "s01");
        //     TestRedisClient(redisClientFactory.CSRedisClient02, "s01");
        //     TestRedisClient(redisClientFactory.CSRedisClient03, "s01");
        //
        //     //监听配置变化后 在进行测试
        //     redisClientFactory.OnRedisChange = () =>
        //     {
        //         TestRedisClient(redisClientFactory.CSRedisClient01, "s02");
        //         TestRedisClient(redisClientFactory.CSRedisClient02, "s02");
        //         TestRedisClient(redisClientFactory.CSRedisClient03, "s02");
        //     };
        // }


        /// <summary>
        /// 获取单例redis 连接字符串
        /// </summary>
        /// <param name="config"></param>
        /// <returns></returns>
        private static string GetRedisSingletonConnectionString(TestConfig config)
        {
            var conStr = config.RedisConfig.First();
            if (conStr.IndexOf("prefix", StringComparison.Ordinal) <= 0)
                conStr += "";
            return conStr;
        }

        /// <summary>
        /// 测试redis client 操作redis 集群
        /// </summary>
        /// <param name="redisClient"></param>
        private static void TestRedisClient(CSRedisClient redisClient, string prefix)
        {
            var key = $"{prefix}:testredisclient";
            redisClient.Set(key, $"{DateTime.Now}-{Guid.NewGuid()}", 300);
            Console.WriteLine(redisClient.Get(key));
            Thread.Sleep(2000);
        }
    }
}