﻿using System.Diagnostics;
using BPA.Component.ClickHouseClient;
using BPA.Component.ClickHouseClientTester.Model;
using BPA.Component.Extensions;
using ClickHouse.Client.ADO;
using ClickHouse.Client.Copy;
using ClickHouse.Client.Utility;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace BPA.Component.ClickHouseClientTester
{
    /// <summary>
    /// ClickHouse.Client
    /// https://github.com/DarkWanderer/ClickHouse.Client
    /// </summary>
    [TestClass()]
    public class ClickHouseHttpClinetTests
    {
        /// <summary>
        /// 最大时间
        /// </summary>
        private readonly DateTime MaxDateTime = DateTime.Parse("2106-01-29 16:40:32");

        /// <summary>
        /// 最小时间
        /// </summary>
        private readonly DateTime MinDateTime = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);

        private readonly ClickHouseConnection clickHouseConnection;

        private readonly long corpId = 10000;

        public ClickHouseHttpClinetTests()
        {
            var clickHouseConStr = "Compression=true;UseSession=false;Timeout=600000;" +
                                   "Database=test;Host=192.168.0.150;Port=8123;Username=default;Password=123456;Protocol=http";

            clickHouseConStr = "Compression=true;UseSession=false;Timeout=600000;" +
                               "Database=statistics;Host=192.168.0.150;Port=8123;Username=default;Password=123456;Protocol=http";

            clickHouseConnection = new ClickHouseConnection(clickHouseConStr);
            clickHouseConnection.Open();
        }

        [TestMethod()]
        public void Test_Insert()
        {
            var bulkRows = new List<object[]>();
            for (var i = 0; i < 10000; i++)
            {
                var row = new CorpInfoSynLocal
                {
                    Id = 1,
                    Name = "t1",
                    No = "n1",
                    Phones = new string[0],
                    CorpId = corpId,
                    CreateDate = DateTime.Now,
                    EnumEmployee = EnumEmployeeStatusType.Normal,
                    IsTTLDelete = false,
                    TTLDate = DateTime.MaxValue,
                    UpdateDate = DateTime.Now
                };
                bulkRows.Add(new object[]
                {
                    row.Id, row.CorpId,
                    row.No, row.Name, row.Phones, row.CreateDate, row.EnumEmployee,
                    row.IsTTLDelete, row.TTLDate, row.UpdateDate
                });
            }

            var columns = "Id, CorpId, No, Name, Phones, CreateDate, EnumEmployee,IsTTLDelete,TTLDate,UpdateDate".Split(",");
            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();
            using var bulkCopy = new ClickHouseBulkCopy(clickHouseConnection)
            {
                DestinationTableName = "test.CorpInfoSynDistributed",
                BatchSize = bulkRows.Count,
            };
            bulkCopy.WriteToServerAsync(bulkRows, columns).GetAwaiter().GetResult();
            stopwatch.Stop();
            Console.WriteLine(stopwatch.ElapsedMilliseconds);
        }

        [TestMethod()]
        public void Test_Delete()
        {
            var bulkRows = new List<object[]>();
            for (var i = 0; i < 1; i++)
            {
                var row = new CorpInfoSynLocal
                {
                    Id = 1,
                    Name = "t1",
                    No = "n1",
                    Phones = new string[0],
                    CorpId = corpId,
                    CreateDate = DateTime.Now,
                    EnumEmployee = EnumEmployeeStatusType.Normal,
                    IsTTLDelete = true,
                    TTLDate = DateTime.Now.AddSeconds(3600),
                    UpdateDate = DateTime.Now
                };
                bulkRows.Add(new object[] {row.Id, row.CorpId, row.IsTTLDelete, row.TTLDate, row.UpdateDate});
            }

            var columns = "Id, CorpId, IsTTLDelete,TTLDate,UpdateDate".Split(",");
            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();
            using var bulkCopy = new ClickHouseBulkCopy(clickHouseConnection)
            {
                DestinationTableName = "test.CorpInfoSynDistributed",
                BatchSize = bulkRows.Count,
            };
            bulkCopy.WriteToServerAsync(bulkRows, columns).GetAwaiter().GetResult();
            stopwatch.Stop();
            Console.WriteLine(stopwatch.ElapsedMilliseconds);
        }


        [TestMethod()]
        public void Test_Queue2()
        {
            var sql = "SELECT   *  FROM  test.CorpInfoSynDistributed    FINAL WHERE Id >{Id:Int64}";
            var cmd = clickHouseConnection.CreateCommand();
            //   cmd.AddParameter("Id", 0L);
            cmd.CommandText = sql;
            cmd.AddParameter("Id", "Int64", 0);
            var oldRows = new Dictionary<long, Dictionary<string, object>>();
            long idx = 0;
            using (var reader = cmd.ExecuteReader())
            {
                while (reader.Read())
                {
                    var row = new Dictionary<string, object>();
                    for (var i = 0; i < reader.FieldCount; i++)
                    {
                        var value = reader[i];
                        var key = reader.GetName(i);
                        row.Add(key, value);
                    }

                    idx++;
                    oldRows.Add(idx, row);
                }
            }
        }

        /// <summary>
        /// sql 注入   参数注入 纯原生
        /// </summary>
        [TestMethod()]
        public void Test_Queue3()
        {
            var sql = "select *  from statistics.CustomerSynLocal where  Name LIKE {name:String} ";
            var cmd = clickHouseConnection.CreateCommand();
            //   cmd.AddParameter("Id", 0L);
            cmd.CommandText = sql;
            cmd.AddParameter("name", "%西%");
            var oldRows = new Dictionary<long, Dictionary<string, object>>();
            long idx = 0;
            using (var reader = cmd.ExecuteReader())
            {
                while (reader.Read())
                {
                    var row = new Dictionary<string, object>();
                    for (var i = 0; i < reader.FieldCount; i++)
                    {
                        var value = reader[i];
                        var key = reader.GetName(i);
                        row.Add(key, value);
                    }

                    idx++;
                    oldRows.Add(idx, row);
                }
            }
        }

        /// <summary>
        /// sql 注入   参数注入 Dapper
        /// </summary>
        [TestMethod()]
        public void Test_Queue4()
        {
            var likeName = "西";
            // likeName = "西 '  or 1=1 --";//sql注入
            var sql = "select Name  from statistics.CustomerSynLocal where  Name LIKE {Name:String} ";
            var param = new Dictionary<string, object>();
            param.Add("Name", $"%{likeName}%"); //参数化， like 用的 % 必须在参数值里面
            var rows = clickHouseConnection.Query<string>(sql, param);
            Console.WriteLine(rows.ToJson());
        }

        /// <summary>
        /// sql 注入   参数注入 Dapper
        /// </summary>
        [TestMethod()]
        public void Test_Queue5()
        {
            var likeName = "西";
            var sql = "select Count(Name)  from statistics.CustomerSynLocal where  Name LIKE {Name:String} ";
            var param = new Dictionary<string, object>();
            param.Add("Name", $"%{likeName}%"); //参数化， like 用的 % 必须在参数值里面

            //异步 obj
            var obj = clickHouseConnection.ExecuteScalarAsync(sql, param).Result;
            Console.WriteLine(obj);

            //异步 泛型
            var count = clickHouseConnection.ExecuteScalarAsync<ulong>(sql, param).Result;
            Console.WriteLine(count);

            //同步 泛型
            count = clickHouseConnection.ExecuteScalar<ulong>(sql, param);
            Console.WriteLine(count);
        }
    }
}