﻿using BPA.Component.DTOCommon.Langs;

namespace BPA.Component.DTOCommon.BaseDTOs
{
    /// <summary>
    /// 分页查询返回结果
    /// </summary>
    /// <typeparam name="TRowDataDTO">数据行泛型</typeparam>
    public class BaseResultPage<TRowDataDTO> : BaseResult<PageData<TRowDataDTO>>
    {
        /// <summary>
        /// BaseResultPage
        /// </summary>
        public BaseResultPage() : base()
        {
            Data = new PageData<TRowDataDTO>();
        }

        /// <summary>
        /// BaseResult
        /// </summary> 
        /// <param name="lang">语言配置</param>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        public BaseResultPage(Lang lang, string moreZh = null, string moreEn = null)
            : base(default, lang, moreZh, moreEn)
        {
        }

        /// <summary>
        /// BaseResult
        /// </summary>
        /// <param name="rows"></param> 
        /// <param name="lang">语言配置</param>
        /// <param name="totalCount"></param>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        public BaseResultPage(List<TRowDataDTO> rows, Lang lang, long totalCount = 0, string moreZh = null,
            string moreEn = null)
            : base(new PageData<TRowDataDTO> {Rows = rows, TotalCount = totalCount}, lang, moreZh, moreEn)
        {
        }

        /// <summary>
        /// OK
        /// </summary>
        /// <param name="rows">业务数据</param>
        /// <param name="totalCount">总共多少条</param> 
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public BaseResultPage<TRowDataDTO> OK(List<TRowDataDTO> rows, long totalCount = 0, string moreZh = null,
            string moreEn = null)
        {
            this.Data = new PageData<TRowDataDTO> {Rows = rows, TotalCount = totalCount};
            base.DoBuild(ConstLangPackge.OK, moreZh, moreEn);
            return this;
        }

        /// <summary>
        /// Fail
        /// </summary>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public new BaseResultPage<TRowDataDTO> Fail(string moreZh = null, string moreEn = null)
        {
            DoBuild(ConstLangPackge.Fail, moreZh, moreEn);
            return this;
        }

        /// <summary>
        /// Build
        /// </summary>
        /// <param name="data">业务数据</param>
        /// <param name="lang">语言配置</param>
        /// <param name="totalCount"></param>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static BaseResultPage<TRowDataDTO> Build(List<TRowDataDTO> data, Lang lang, long totalCount = 0,
            string moreZh = null, string moreEn = null)
            => new BaseResultPage<TRowDataDTO>(data, lang, totalCount, moreZh, moreEn);

        /// <summary>
        /// Build
        /// </summary>
        /// <param name="lang">语言配置</param>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static new BaseResultPage<TRowDataDTO> Build(Lang lang, string moreZh = null, string moreEn = null)
            => new BaseResultPage<TRowDataDTO>(default, lang, 0, moreZh, moreEn);

        /// <summary>
        /// OK
        /// </summary>
        /// <param name="data">业务数据</param>
        /// <param name="totalCount">总共多少条</param> 
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static BaseResultPage<TRowDataDTO> BuildOK(List<TRowDataDTO> data, long totalCount = 0,
            string moreZh = null, string moreEn = null)
            => new BaseResultPage<TRowDataDTO>().OK(data, totalCount, moreZh, moreEn);

        /// <summary>
        /// BuildFail
        /// </summary>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static new BaseResultPage<TRowDataDTO> BuildFail(string moreZh = null, string moreEn = null)
            => new BaseResultPage<TRowDataDTO>(default, ConstLangPackge.Fail, 0, moreZh, moreEn);
    }

    /// <summary>
    /// 自定义分页查询返回结果
    /// </summary>
    /// <typeparam name="TCustomPageData"></typeparam>
    /// <typeparam name="TRowDataDTO">数据行泛型</typeparam>
    public class BaseCustomResultPage<TCustomPageData, TRowDataDTO> : BaseResult<TCustomPageData>
        where TCustomPageData : PageData<TRowDataDTO>, new()
    {
        /// <summary>
        /// BaseResultPage
        /// </summary>
        public BaseCustomResultPage()
        {
        }

        /// <summary>
        /// BaseResult
        /// </summary> 
        /// <param name="lang">语言配置</param>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        public BaseCustomResultPage(Lang lang, string moreZh = null, string moreEn = null)
            : base(default, lang, moreZh, moreEn)
        {
        }

        /// <summary>
        /// Fail
        /// </summary>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public new BaseCustomResultPage<TCustomPageData, TRowDataDTO> Fail(string moreZh = null, string moreEn = null)
        {
            DoBuild(ConstLangPackge.Fail, moreZh, moreEn);
            return this;
        }

        /// <summary>
        /// OK
        /// </summary>
        /// <param name="pageData">业务数据</param> 
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public new BaseCustomResultPage<TCustomPageData, TRowDataDTO> OK(TCustomPageData pageData, string moreZh = null,
            string moreEn = null)
        {
            this.Data = pageData;
            base.DoBuild(ConstLangPackge.OK, moreZh, moreEn);
            return this;
        }


        /// <summary>
        /// OK
        /// </summary>
        /// <param name="pageData">业务数据</param> 
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static new BaseCustomResultPage<TCustomPageData, TRowDataDTO> BuildOK(TCustomPageData pageData,
            string moreZh = null, string moreEn = null)
            => new BaseCustomResultPage<TCustomPageData, TRowDataDTO>().OK(pageData, moreZh, moreEn);

        /// <summary>
        /// BuildFail
        /// </summary>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static new BaseCustomResultPage<TCustomPageData, TRowDataDTO> BuildFail(string moreZh = null,
            string moreEn = null)
            => new BaseCustomResultPage<TCustomPageData, TRowDataDTO>().Fail(moreZh, moreEn);
    }

    /// <summary>
    ///  业务数据
    /// </summary>
    /// <typeparam name="TRow"></typeparam>
    public class PageData<TRow>
    {
        /// <summary>
        /// 数据行
        /// </summary>
        public List<TRow> Rows { set; get; }

        /// <summary>
        /// 总共条数
        /// </summary>
        public long TotalCount { set; get; }
    }
}