﻿using BPA.Component.DTOCommon.Langs;
using Microsoft.AspNetCore.Http;

namespace BPA.Component.DTOCommon.BaseDTOs
{
    /// <summary>
    /// BaseRsp
    /// </summary>
    /// <typeparam name="ResultBusinessData"></typeparam>
    public class BaseResult<ResultBusinessData> : BaseResult
    {
        /// <summary>
        /// 业务数据
        /// </summary>
        public ResultBusinessData Data { set; get; }

        /// <summary>
        /// BaseResult
        /// </summary>
        public BaseResult() : base()
        {
        }

        /// <summary>
        /// BaseResult
        /// </summary>
        /// <param name="data">业务数据</param>
        /// <param name="lang">语言配置</param>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        public BaseResult(ResultBusinessData data, Lang lang, string moreZh = null, string moreEn = null) : base(lang,
            moreZh, moreEn)
        {
            this.Data = data;
        }

        /// <summary>
        /// BaseResult
        /// </summary>
        /// <param name="lang">语言配置</param>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        public BaseResult(Lang lang, string moreZh = null, string moreEn = null) : base(lang, moreZh, moreEn)
        {
        }

        /// <summary>
        /// OK
        /// </summary>
        /// <param name="data">业务数据</param> 
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public BaseResult<ResultBusinessData> OK(ResultBusinessData data, string moreZh = null, string moreEn = null)
        {
            this.Data = data;
            base.DoBuild(ConstLangPackge.OK, moreZh, moreEn);
            return this;
        }

        /// <summary>
        /// Fail
        /// </summary>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public new BaseResult<ResultBusinessData> Fail(string moreZh = null, string moreEn = null)
        {
            base.DoBuild(ConstLangPackge.Fail, moreZh, moreEn);
            return this;
        }

        /// <summary>
        /// Build
        /// </summary>
        /// <param name="data">业务数据</param>
        /// <param name="lang">语言配置</param>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static BaseResult<ResultBusinessData> Build(
            ResultBusinessData data,
            Lang lang,
            string moreZh = null,
            string moreEn = null)
            => new BaseResult<ResultBusinessData>(data, lang, moreZh, moreEn);

        /// <summary>
        /// Build
        /// </summary>
        /// <param name="lang">语言配置</param>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static new BaseResult<ResultBusinessData> Build(Lang lang, string moreZh = null, string moreEn = null)
            => new BaseResult<ResultBusinessData>(default, lang, moreZh, moreEn);

        /// <summary>
        /// BuildOK
        /// </summary>
        /// <param name="data">业务数据</param> 
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static BaseResult<ResultBusinessData> BuildOK(ResultBusinessData data, string moreZh = null,
            string moreEn = null)
            => new BaseResult<ResultBusinessData>(data, ConstLangPackge.OK, moreZh, moreEn);

        /// <summary>
        /// BuildFail
        /// </summary>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static new BaseResult<ResultBusinessData> BuildFail(string moreZh = null, string moreEn = null)
            => new BaseResult<ResultBusinessData>(default, ConstLangPackge.Fail, moreZh, moreEn);
    }
}