﻿using BPA.Component.DTOCommon.Langs;

namespace BPA.Component.DTOCommon.BaseDTOs
{
    /// <summary>
    /// 业务异常
    /// </summary>
    public class BusinessException : Exception
    {
        /// <summary>
        /// 异常消息
        /// </summary>
        private readonly string expMsg;

        /// <summary>
        /// 返回代码
        /// </summary>
        public Lang Lang { set; get; }

        /// <summary>
        /// 更多的中文提示,返回给接口用方
        /// </summary>
        public string MoreZhMsg { set; get; }

        /// <summary>
        /// 更多的英文提示,返回给接口用方
        /// </summary>
        public string MoreEnMsg { set; get; }

        /// <summary>
        /// 异常消息
        /// </summary>
        public override string Message
        {
            get => expMsg;
        }

        /// <summary>
        /// 构造方法
        /// </summary>
        /// <param name="lang">语音包</param>
        /// <param name="expMessage">业务异常消息,记录日志用</param>
        public BusinessException(Lang lang, string expMessage = null)
        {
            Lang = lang;
            this.expMsg = $"[{Lang.Key}],{Lang.Zh}";
            if (!string.IsNullOrEmpty(MoreZhMsg))
                this.expMsg += $",{MoreZhMsg}";
            if (!string.IsNullOrEmpty(MoreEnMsg))
                this.expMsg += $",{MoreEnMsg}";
            if (!string.IsNullOrEmpty(expMessage))
                this.expMsg += $",{expMessage}";
        }

        /// <summary>
        /// 构造方法
        /// </summary>
        /// <param name="result"></param>
        /// <param name="expMessage"></param>
        public BusinessException(BaseResult result, string expMessage = null)
        {
            Lang = result.Lang;
            this.expMsg = expMessage;
            this.MoreZhMsg = result.MoreZhMsg;
            this.MoreEnMsg = result.MoreEnMsg;
            this.expMsg = $"[{Lang.Key}],{Lang.Zh}";
            if (!string.IsNullOrEmpty(MoreZhMsg))
                this.expMsg += $",{MoreZhMsg}";
            if (!string.IsNullOrEmpty(MoreEnMsg))
                this.expMsg += $",{MoreEnMsg}";
            if (!string.IsNullOrEmpty(expMessage))
                this.expMsg += $",{expMessage}";
        }

        /// <summary>
        /// 构造方法
        /// </summary>
        /// <param name="lang">语音包</param>
        /// <param name="moreZh"></param>
        /// <param name="moreEn"></param>
        /// <param name="expMessage">业务异常消息,记录日志用</param>
        public BusinessException(Lang lang, string moreZh = null, string moreEn = null, string expMessage = null)
        {
            Lang = lang;
            this.MoreZhMsg = moreZh;
            this.MoreEnMsg = moreEn;
            this.expMsg = $"[{Lang.Key}],{Lang.Zh}";
            if (!string.IsNullOrEmpty(MoreZhMsg))
                this.expMsg += $",{MoreZhMsg}";
            if (!string.IsNullOrEmpty(MoreEnMsg))
                this.expMsg += $",{MoreEnMsg}";
            if (!string.IsNullOrEmpty(expMessage))
                this.expMsg += $",{expMessage}";
        }

        /// <summary>
        ///  设置更多的中文提示,返回给接口调用方
        /// </summary>
        /// <param name="moreZh"></param>
        /// <returns></returns>
        public BusinessException MoreZh(string moreZh)
        {
            MoreZhMsg = moreZh;
            return this;
        }

        /// <summary>
        ///  设置更多的英文提示,返回给接口调用方
        /// </summary>
        /// <param name="moreEn"></param>
        /// <returns></returns>
        public BusinessException MoreEn(string moreEn)
        {
            MoreEnMsg = moreEn;
            return this;
        }

        /// <summary>
        /// 操作失败业务异常
        /// </summary>
        /// <param name="message">业务异常消息,记录日志用</param>
        /// <returns></returns>
        public static BusinessException Fail(string message = null) =>
            new BusinessException(ConstLangPackge.Fail, message);
    }
}