﻿using Newtonsoft.Json;
using Newtonsoft.Json.Converters;

namespace BPA.Component.DTOCommon.JsonConverters
{
    /// <summary>
    /// conver DateTime.MinValue to string.empty for js;
    /// 序列化工具辅助类，用于将DateTime类型的主键的值==DateTime.MinValue 转化为 空字符串 
    /// </summary>
    public class JsDateTimeConverter : IsoDateTimeConverter
    {
        private static readonly string defaultDateTimeFormat = "yyyy-MM-dd HH:mm:ss";

        private static readonly DateTime date1970 = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Local);

        /// <summary>
        /// Writes the JSON representation of the object.
        /// </summary>
        /// <param name="writer">The <see cref="JsonWriter"/> to write to.</param>
        /// <param name="value">The value.</param>
        /// <param name="serializer">The calling serializer.</param>
        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
            if (value is DateTime dateTime)
            {
                if (dateTime == DateTime.MinValue || dateTime == date1970)
                {
                    writer.WriteValue(string.Empty);
                    return;
                }

                writer.WriteValue(dateTime.ToString(defaultDateTimeFormat, Culture));
                return;
            }

            if (value is DateTimeOffset dateTimeOffSet)
            {
                if (dateTimeOffSet == DateTime.MinValue || dateTimeOffSet == date1970)
                {
                    writer.WriteValue(string.Empty);
                    return;
                }

                writer.WriteValue(dateTimeOffSet.ToString(defaultDateTimeFormat, Culture));
                return;
            }

            base.WriteJson(writer, value, serializer);
        }

        /// <summary>
        /// Reads the JSON representation of the object.
        /// </summary>
        /// <param name="reader">The <see cref="JsonReader"/> to read from.</param>
        /// <param name="objectType">Type of the object.</param>
        /// <param name="existingValue">The existing value of object being read.</param>
        /// <param name="serializer">The calling serializer.</param>
        /// <returns>The object value.</returns>
        public override object ReadJson(JsonReader reader, Type objectType, object existingValue,
            JsonSerializer serializer)
        {
            if (objectType == typeof(DateTime))
            {
                if (reader.Value?.ToString() == string.Empty)
                {
                    return DateTime.MinValue;
                }
            }

            if (objectType == typeof(DateTimeOffset))
            {
                if (reader.Value?.ToString() == string.Empty)
                {
                    return DateTime.MinValue;
                }
            }

            return base.ReadJson(reader, objectType, existingValue, serializer);
        }
    }
}