﻿using Newtonsoft.Json;

namespace BPA.Component.DTOCommon.JsonConverters
{
    /// <summary>
    /// conver  long to string for js;
    /// 序列化工具辅助类，用于将long类型的主键值 转化为 字符串 
    /// js不支持18位的数字,必须转换为js
    /// </summary>
    public class JsUlongConverter : JsonConverter
    {
        private static Type ulongType = typeof(ulong);
        private static Type ulongNullType = typeof(ulong?);
        private static Type longType = typeof(long);
        private static Type longNullType = typeof(long?);

        /// <summary>
        /// long类型
        /// </summary>
        private static List<Type> longTypes = new List<Type> {longType, longNullType};

        /// <summary>
        /// 可以转换的类型
        /// </summary>
        private static List<Type> canConvertTypes = new List<Type> {ulongType, ulongNullType, longType, longNullType};

        /// <summary>
        ///  判断属性是否能做转换
        /// </summary>
        /// <param name="objectType"></param>
        /// <returns></returns>
        public override bool CanConvert(Type objectType) => canConvertTypes.Contains(objectType);


        /// <summary>
        /// 将字符串 解密，还原为 long类型的字符串。
        /// </summary>
        /// <param name="reader">属性反序列化值</param>
        /// <param name="objectType">属性类型</param>
        /// <param name="existingValue"></param>
        /// <param name="serializer">序列化工具类</param>
        /// <returns></returns>
        public override object ReadJson(JsonReader reader, Type objectType, object existingValue,
            JsonSerializer serializer)
        {
            if (reader.TokenType == JsonToken.Null)
                return null;
            if (reader == null)
                return null;
            try
            {
                if (reader.TokenType == JsonToken.String) // 转换单列
                {
                    string value = reader.Value.ToString();
                    if (string.IsNullOrWhiteSpace(value))
                        return null;
                    if (longTypes.Contains(objectType))
                        return Convert.ToInt64(value);
                    return Convert.ToUInt64(value);
                }
                else if (reader.TokenType == JsonToken.Integer)
                {
                    if (longTypes.Contains(objectType))
                        return Convert.ToInt64(reader.Value);
                    return Convert.ToUInt64(reader.Value);
                }

                if (longTypes.Contains(objectType))
                    return Convert.ToInt64(reader.Value);
                return Convert.ToUInt64(reader.Value);
            }
            catch (Exception ex)
            {
                throw new Exception(
                    string.Format(" {0} -- Error converting value {1} to type '{2}'", reader.Path, reader.Value,
                        objectType), ex);
            }

            throw new Exception(string.Format(" {0} --Unexpected token {1} when parsing enum", reader.Path,
                reader.TokenType));
        }

        /// <summary>
        ///  将 long类型的值 转换为字符串值
        /// </summary>
        /// <param name="writer">转出的字符串字节流</param>
        /// <param name="value">待转换的值</param>
        /// <param name="serializer">序列化工具类</param>
        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
            if (value == null)
            {
                writer.WriteNull();
                return;
            }

            writer.WriteValue(value.ToString());
        }
    }
}