﻿namespace BPA.Component.DTOCommon.Langs
{
    /// <summary>
    /// 语言基础包
    /// </summary>
    public abstract class BaseLangPackge
    {
        /// <summary>
        /// 语言字段
        /// </summary>
        private readonly Dictionary<string, Lang> langDic = new Dictionary<string, Lang>();

        /// <summary>
        /// 处理成功
        /// </summary>
        public readonly Lang OK = ConstLangPackge.OK;

        /// <summary>
        /// 处理失败
        /// </summary>
        public readonly Lang Fail = ConstLangPackge.Fail;

        /// <summary>
        /// 处理错误
        /// </summary>
        public readonly Lang Error = ConstLangPackge.Error;

        /// <summary>
        /// 登陆过期
        /// </summary>
        public readonly Lang LoginExpired = ConstLangPackge.LoginExpired;

        /// <summary>
        /// 无权限
        /// </summary>
        public readonly Lang Unauthorized = ConstLangPackge.Unauthorized;

        /// <summary>
        /// 参数错误
        /// </summary>
        public readonly Lang ParamInvalid = ConstLangPackge.ParamInvalid;

        /// <summary>
        /// 请求太频繁
        /// </summary>
        public static Lang RequestLimit = ConstLangPackge.RequestLimit;

        /// <summary>
        /// 构造方法
        /// </summary>
        public BaseLangPackge()
        {
            var fields = GetType().GetFields();
            foreach (var field in fields)
            {
                if (!(field.GetValue(this) is Lang lang))
                {
                    Console.WriteLine($"*base lang packge* warning : propery [{field.Name}] is not Lang!");
                    continue;
                }

                if (lang.Key != field.Name)
                    throw new Exception($"*base lang packge* error: [{this.GetType().FullName}] lang key [{lang.Key}] 不等于 [{field.Name}]");
                if (langDic.ContainsKey(lang.Key))
                    throw new Exception($"*base lang packge* error: [{this.GetType().FullName}] lang key [{lang.Key}] 重复");
                langDic.Add(lang.Key, lang);
            }
        }

        /// <summary>
        /// 索引,根据LangKey获取Lang对象,获取不到时返回null
        /// </summary>
        /// <param name="langKey">langKey</param>
        /// <returns></returns>
        public Lang this[string langKey]
        {
            get => langDic.TryGetValue(langKey, out Lang lang) ? lang : null;
        }
    }
}
