﻿using Newtonsoft.Json;

namespace BPA.Component.DTOCommon.Langs
{
    /// <summary>
    /// 语音包
    /// </summary>
    public class Lang
    {
        /// <summary>
        /// Key
        /// </summary>
        [JsonProperty]
        public string Key { private set; get; }

        /// <summary>
        /// 中文
        /// </summary>
        [JsonProperty]
        public string Zh { private set; get; }

        /// <summary>
        /// 英文
        /// </summary>
        [JsonProperty]
        public string En { private set; get; }

        /// <summary>
        /// 语言
        /// </summary>
        public Lang()
        {
        }

        /// <summary>
        /// Lang
        /// </summary>
        /// <param name="key"></param>
        /// <param name="zh"></param>
        /// <param name="en"></param>
        public Lang(string key, string zh, string en)
        {
            this.Key = key;
            this.Zh = zh;
            this.En = en;
        }

        /// <summary>
        /// 索引,根据类型语言类型获取对应的值
        /// </summary>
        /// <param name="langType"></param>
        /// <returns></returns>
        public string this[LangType langType]
        {
            get => langType == LangType.Zn ? this.Zh : this.En;
        }

        /// <summary>
        /// Build
        /// </summary>
        /// <param name="key"></param>
        /// <param name="zh"></param>
        /// <param name="en"></param>
        /// <returns></returns>
        public static Lang Build(string key, string zh, string en) => new Lang(key, zh, en);

        /// <summary>
        /// ToString
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return $"[{Key}]:{Zh},{En}";
        }
    }
}
