﻿using System.ComponentModel.DataAnnotations;

namespace BPA.Component.DTOCommon.Validations
{
    /// <summary>
    /// 验证最大长度
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field | AttributeTargets.Parameter, AllowMultiple = false)]
    public class IbpaMaxLengthAttribute : MaxLengthAttribute, BPAValidationAttribute
    {
        /// <summary>
        /// 中文提示,返回给接口用方
        /// </summary>
        public string ZhMessage { set; get; }

        /// <summary>
        /// 英文提示,返回给接口用方
        /// </summary>
        public string EnMessage { set; get; }

        /// <summary>
        /// 验证参数最大长度
        /// </summary> 
        /// <param name="length">最大长度</param>
        /// <param name="fieldZhName">字段中文名</param>
        /// <param name="fieldEnName">字段英文名</param>
        public IbpaMaxLengthAttribute(int length, string fieldZhName, string fieldEnName) : base(length)
        {
            this.ZhMessage = $"{fieldZhName}长度不能超过{length}";
            this.EnMessage = $"{fieldEnName} length cannot be longer than {length}";
        }
    }
}
