﻿using System.ComponentModel.DataAnnotations;

namespace BPA.Component.DTOCommon.Validations
{
    /// <summary>
    /// 验证字符串长度范围
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field | AttributeTargets.Parameter, AllowMultiple = false)]
    public class StringLengthAttribute : System.ComponentModel.DataAnnotations.StringLengthAttribute, BPAValidationAttribute
    {
        /// <summary>
        /// 中文提示,返回给接口用方
        /// </summary>
        public string ZhMessage { set; get; }

        /// <summary>
        /// 英文提示,返回给接口用方
        /// </summary>
        public string EnMessage { set; get; }

        /// <summary>
        /// 验证参数长度
        /// </summary> 
        /// <param name="minLength">最小长度</param>
        /// <param name="maxiLength">最大长度</param>
        /// <param name="fieldZhName">字段中文名</param>
        /// <param name="fieldEnName">字段英文名</param>
        public StringLengthAttribute(int minLength, int maxiLength, string fieldZhName, string fieldEnName) : base(maxiLength)
        {
            this.MinimumLength = minLength;
            this.ZhMessage = $"{fieldZhName}长度在{minLength}和{maxiLength}之间";
            this.EnMessage = $"{fieldEnName} length must be between {minLength} and {maxiLength}";
        }
    }
}
