﻿using System.Collections.Generic;
using System.Threading.Tasks;
using BPA.Component.DbClient.RepositoryModel;
using BPA.Component.DbClientTester.Entities;
using BPA.Component.Extensions;
using Microsoft.Extensions.Logging;

namespace BPA.Component.DbClientTester.Repositorys
{
    /// <summary>
    /// 用户仓储
    /// </summary>
    public class UserRepository : BaseDbRepository<TestDbClient, BPA_Users>, IUserRepository
    {
        private readonly ILogger<UserRepository> _logger;

        /// <summary>
        /// 构造方法注入 SqlSugarClient
        /// </summary>
        /// <param name="dbClient"></param>
        /// <param name="logger"></param>
        public UserRepository(TestDbClient dbClient, ILogger<UserRepository> logger) : base(dbClient)
        {
            _logger = logger;
        }

        /// <summary>
        /// 查询
        /// </summary>
        /// <param name="id"></param>
        /// <param name="name"></param>
        /// <returns></returns>
        public async Task<List<BPA_Users>> Query(string id, string name)
        {
            var result = await DbClient.Queryable<BPA_Users>()
                .WhereIF(!string.IsNullOrEmpty(id), x => x.Id == id)
                .WhereIF(!string.IsNullOrEmpty(name), x => x.Name.Contains(name))
                .ToListAsync();
            if (result != null) _logger.LogInformation(result.ToJson());
            return result;
        }
    }
}