﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using BPA.Component.ApolloClient;
using BPA.Component.DbClientTester.Entities;
using BPA.Component.DbClientTester.Repositorys;
using BPA.Component.Extensions;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;

namespace BPA.Component.DbClientTester
{
    /// <summary>
    /// 测试仓储模式
    /// </summary>
    public static class TestRepositoryModel
    {
        public static async Task Test()
        {
            await TestUseOne();
        }

        /// <summary>
        /// 测试单个数据库
        /// </summary>
        private static async Task TestUseOne()
        {
            var builder = new ConfigurationBuilder();
            builder.AddEnvironmentVariables();
            var configuration = builder.Build();
            configuration.AddApolloConfiguration();

            var services = new ServiceCollection();
            services.AddApollo<TestConfig>();
            services.AddLogging(configure => { configure.AddConsole(); });
            services.AddScoped<TestDbClient>();
            services.AddTransient<IUserRepository, UserRepository>();

            //构建容器提供对象
            var serviceProvider = services.BuildServiceProvider();
            var testDbClient = serviceProvider.GetService<TestDbClient>();
            if (testDbClient == null) return;
            testDbClient.DbMaintenance.GetTableInfoList();

            //从容器中获取IUserRepository 对象 
            var userRepository = serviceProvider.GetService<IUserRepository>() ??
                                 throw new ArgumentNullException($"serviceProvider.GetService<IUserRepository>()");

            var id = Guid.NewGuid().ToString();

            // 插入数据
            await userRepository.InsertAsync(new BPA_Users
            {
                Id = id,
                Name = "stevelee test",
                Account = "aabbcc",
                AdminType = 1,
                Password = "123",
                CreateBy = "abvc",
                CreateAt = DateTime.Now
            });

            // linq first
            var f = await userRepository.FirstAsync(x => x.Id != "", x => new {x.Name, x.Id});
            Console.WriteLine(JsonConvert.SerializeObject(f));

            // linq first
            var f2 = await userRepository.FirstAsync(x => x.Id == id);
            Console.WriteLine(JsonConvert.SerializeObject(f2));

            // linq queryable
            var queryable = await userRepository.SimpleClient.AsQueryable().Where(x => x.Id != "").ToListAsync();
            Console.WriteLine(JsonConvert.SerializeObject(queryable));

            // query
            var users = await userRepository.Query(null, null);
            Console.WriteLine(JsonConvert.SerializeObject(users));

            // query
            users = await userRepository.Query(null, "ha");
            Console.WriteLine(JsonConvert.SerializeObject(users));

            // 删除数据
            var r = await userRepository.DeleteAsync(x => x.Id == id);
            Console.WriteLine(r);
        }
    }
}