﻿using System.Security.Cryptography;
using System.Text;

namespace BPA.Component.Extensions
{
    /// <summary>
    /// AES扩展类
    /// </summary>
    public static class AESExtension
    {
        /// <summary>
        /// AES加密,CipherMode.ECB,PaddingMode.PKCS7
        /// </summary>
        /// <param name="plainText">明文</param>
        /// <param name="aesKey">32位秘钥</param>
        /// <param name="mode">加密模式,默认ECB,   1.电码本模式（Electronic Codebook Book (ECB)）；2.密码分组链接模式（Cipher Block Chaining (CBC)）；3.计算器模式（Counter (CTR)）；4.密码反馈模式（Cipher FeedBack (CFB)）；5.输出反馈模式（Output FeedBack (OFB)）</param>
        /// <param name="padding">填充模式,默认PKCS7</param>
        /// <returns></returns>
        public static string AesEncrypt(this string plainText, string aesKey, CipherMode mode = CipherMode.ECB,
            PaddingMode padding = PaddingMode.PKCS7)
        {
            if (string.IsNullOrEmpty(plainText)) return null;
            Byte[] toEncryptArray = Encoding.UTF8.GetBytes(plainText);
#pragma warning disable 618
            RijndaelManaged rm = new RijndaelManaged
#pragma warning restore 618
            {
                Key = Encoding.UTF8.GetBytes(aesKey),
                Mode = mode,
                Padding = padding
            };
            ICryptoTransform cTransform = rm.CreateEncryptor();
            var resultArray = cTransform.TransformFinalBlock(toEncryptArray, 0, toEncryptArray.Length);
            return Convert.ToBase64String(resultArray, 0, resultArray.Length);
        }

        /// <summary>
        /// AES解密
        /// </summary>
        /// <param name="text">密文</param>
        /// <param name="aesKey">32位秘钥</param>
        /// <param name="mode">加密模式,默认ECB,   1.电码本模式（Electronic Codebook Book (ECB)）；2.密码分组链接模式（Cipher Block Chaining (CBC)）；3.计算器模式（Counter (CTR)）；4.密码反馈模式（Cipher FeedBack (CFB)）；5.输出反馈模式（Output FeedBack (OFB)）</param>
        /// <param name="padding">填充模式,默认PKCS7</param>
        /// <returns></returns>
        public static string AesDecrypt(this string text, string aesKey, CipherMode mode = CipherMode.ECB,
            PaddingMode padding = PaddingMode.PKCS7)
        {
            if (string.IsNullOrEmpty(text)) return null;
            var toEncryptArray = Convert.FromBase64String(text);
            RijndaelManaged rm = new RijndaelManaged
            {
                Key = Encoding.UTF8.GetBytes(aesKey),
                Mode = mode,
                Padding = padding
            };
            ICryptoTransform cTransform = rm.CreateDecryptor();
            var resultArray = cTransform.TransformFinalBlock(toEncryptArray, 0, toEncryptArray.Length);
            return Encoding.UTF8.GetString(resultArray);
        }
    }
}