﻿namespace BPA.Component.Extensions
{
    /// <summary>
    /// 检查是否为空扩展类
    /// </summary>
    public static class CheckNullExtension
    {
        /// <summary>
        /// 检查空,如果为空，抛出参数空异常
        /// </summary>
        /// <typeparam name="Tsource"></typeparam>
        /// <param name="source"></param>
        /// <param name="nameof">抛出异常时的参数名或者异常消息</param>
        /// <returns> throw new ArgumentNullException(nameof);</returns>
        public static Tsource CheckNull<Tsource>(this Tsource source, string nameof)
        {
            if (source == null)
                throw new ArgumentNullException(nameof);
            return source;
        }

        /// <summary>
        /// 检查空或空字符串,如果为空，抛出参数空异常
        /// </summary> 
        /// <param name="source"></param>
        /// <param name="nameof">抛出异常时的参数名或者异常消息</param>
        /// <returns> throw new ArgumentNullException(nameof);</returns>
        public static string CheckNull(this string source, string nameof)
        {
            if (source.IsEmpty())
                throw new ArgumentNullException(nameof);
            return source;
        }

        /// <summary>
        /// 判断是否为空,如果是string将根据IsEmpty()判断null或者空字符串
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
        public static bool IsNull<TSource>(this TSource source)
        {
            if (source == null) return true;
            if (source is string)
                return (source as string).IsEmpty();
            return source == null;
        }

        /// <summary>
        /// 判断对象不为空
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
        public static bool IsNotNull<TSource>(this TSource source)
        {
            return !source.IsNull();
        }
    }
}