﻿using Newtonsoft.Json;

namespace BPA.Component.Extensions
{
    /// <summary>
    /// JsonExtension
    /// </summary>
    public static class JsonExtension
    {
        /// <summary>
        /// obj 转 json 字符串
        /// </summary>
        /// <typeparam name="TObj">需要转成json格式字符的对象类型</typeparam>
        /// <param name="source"></param>
        /// <param name="settings">配置,默认为null,如果为空将使用默认配置或者全局配置好的</param>
        /// <returns></returns>
        public static string ToJson<TObj>(this TObj source, JsonSerializerSettings settings = null!) =>
            JsonConvert.SerializeObject(source, settings);

        /// <summary>
        /// obj 转 json 字符串,不会抛出异常
        /// </summary>
        /// <typeparam name="TObj">需要转成json格式字符的对象类型</typeparam>
        /// <param name="source"></param>
        /// <param name="defaultJson">转换失败时返回的默认值</param>
        /// <param name="settings">配置,默认为null,如果为空将使用默认配置或者全局配置好的</param>
        /// <returns></returns>
        public static string TryToJson<TObj>(this TObj source, string? defaultJson = "",
            JsonSerializerSettings settings = null!)
        {
            try
            {
                return source.ToJson(settings);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                return defaultJson;
            }
        }

        /// <summary>
        /// json 字符串 转 obj
        /// </summary>
        /// <typeparam name="TObj">需要转成json格式字符的对象类型</typeparam>
        /// <param name="source"></param>
        /// <param name="settings">配置,默认为null,如果为空将使用默认配置或者全局配置好的</param>
        /// <returns></returns>
        public static TObj? JsonToObj<TObj>(this string source, JsonSerializerSettings settings = null!)
        {
            return JsonConvert.DeserializeObject<TObj>(source, settings);
        }

        /// <summary>
        /// json 字符串 转 obj,不会抛出异常
        /// </summary>
        /// <typeparam name="TObj">需要转成json格式字符的对象类型</typeparam>
        /// <param name="source"></param>
        /// <param name="defaultObj">转换失败时返回的默认值</param>
        /// <param name="settings">配置,默认为null,如果为空将使用默认配置或者全局配置好的</param>
        /// <returns></returns>
        public static TObj? JsonTryToObj<TObj>(this string source, TObj defaultObj = default,
            JsonSerializerSettings settings = null!)
        {
            try
            {
                return source.JsonToObj<TObj>(settings);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                return defaultObj;
            }
        }
    }
}