﻿using AutoMapper;

namespace BPA.Component.Extensions;

/// <summary>
/// 
/// </summary>
public static class MapperExtension
{
    /// <summary>
    /// 类型映射
    /// </summary>
    /// <typeparam name="TDestination">转化之后的model，可以理解为viewmodel</typeparam>
    /// <typeparam name="TSource">要被转化的实体，Entity</typeparam>
    /// <param name="source">可以使用这个扩展方法的类型，任何引用类型</param>
    /// <returns>转化之后的实体</returns>
    public static TDestination MapTo<TDestination, TSource>(this TSource source)
        where TDestination : class
        where TSource : class
    {
#pragma warning disable CS8603
        return source == null
            ? default
            : new MapperConfiguration(cfg => cfg.CreateMap<TSource, TDestination>())
                .CreateMapper()
                .Map<TDestination>(source);
#pragma warning restore CS8603
    }

    /// <summary>
    /// 集合列表类型映射
    /// </summary>
    /// <typeparam name="TDestination">转化之后的model，可以理解为viewmodel</typeparam>
    /// <typeparam name="TSource">要被转化的实体，Entity</typeparam>
    /// <param name="source">可以使用这个扩展方法的类型，任何引用类型</param>
    /// <returns>转化之后的实体列表</returns>
    public static IEnumerable<TDestination> MapToList<TDestination, TSource>(this IEnumerable<TSource> source)
        where TDestination : class
        where TSource : class
    {
        return source == null
            ? new List<TDestination>()
            : new MapperConfiguration(cfg => cfg.CreateMap<TSource, TDestination>())
                .CreateMapper()
                .Map<List<TDestination>>(source);
    }
}