﻿using Newtonsoft.Json;

namespace BPA.Component.Extensions
{
    /// <summary>
    /// ObjectExtension
    /// </summary>
    public static class ObjectExtension
    {
        /// <summary>
        /// 是否在范围内,开区间，start&lt;source&lt;end
        /// </summary>
        /// <param name="source"></param>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        public static bool IsInRange(this int source, int start, int end)
        {
            return source > start && source < end;
        }

        /// <summary>
        /// 是否在范围内,闭区间 start &lt;= source &lt;= end
        /// </summary>
        /// <param name="source"></param>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        public static bool IsBetween(this int source, int start, int end)
        {
            return source >= start && source <= end;
        }

        /// <summary>
        /// 是否在范围内,开区间 ，start&lt;source&lt;end
        /// </summary>
        /// <param name="source"></param>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        public static bool IsInRange(this float source, float start, float end)
        {
            return source > start && source < end;
        }

        /// <summary>
        /// 是否在范围内,闭区间 ，start &lt;= source &lt;= end
        /// </summary>
        /// <param name="source"></param>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        public static bool IsBetween(this float source, float start, float end)
        {
            return source >= start && source <= end;
        }

        /// <summary>
        /// 是否在范围内,开区间 ，start&lt;source&lt;end
        /// </summary>
        /// <param name="source"></param>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        public static bool IsInRange(this long source, long start, long end)
        {
            return source > start && source < end;
        }

        /// <summary>
        /// 是否在范围内,闭区间 ，start &lt;= source &lt;= end
        /// </summary>
        /// <param name="source"></param>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        public static bool IsBetween(this long source, long start, long end)
        {
            return source >= start && source <= end;
        }

        /// <summary>
        /// 是否在范围内,开区间 ，start&lt;source&lt;end
        /// </summary>
        /// <param name="source"></param>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        public static bool IsInRange(this decimal source, decimal start, decimal end)
        {
            return source > start && source < end;
        }

        /// <summary>
        /// 是否在范围内,闭区间 ，start &lt;= source &lt;= end
        /// </summary>
        /// <param name="source"></param>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        public static bool IsBetween(this decimal source, decimal start, decimal end)
        {
            return source >= start && source <= end;
        }

        /// <summary>
        /// 是否在范围内,开区间 ，start&lt;source&lt;end
        /// </summary>
        /// <param name="source"></param>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        public static bool IsInRange(this DateTime source, DateTime start, DateTime end)
        {
            return source > start && source < end;
        }

        /// <summary>
        /// 是否在范围内,闭区间 ，start &lt;= source &lt;= end
        /// </summary>
        /// <param name="source"></param>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        public static bool IsBetween(this DateTime source, DateTime start, DateTime end)
        {
            return source >= start && source <= end;
        }

        /// <summary>
        /// 是否在范围内,开区间 ，start&lt;source&lt;end
        /// </summary>
        /// <param name="source"></param>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        public static bool IsInRange(this short source, short start, short end)
        {
            return source > start && source < end;
        }

        /// <summary>
        /// 是否在范围内,闭区间 start &lt;= source &lt;= end
        /// </summary>
        /// <param name="source"></param>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        public static bool IsBetween(this short source, short start, short end)
        {
            return source >= start && source <= end;
        }

        /// <summary>
        /// 参数数组里面是否存在 source,例如:"aaa".IsIn("aa", "bb","aaa")= true
        /// </summary>
        /// <param name="source"></param>
        /// <param name="sources"></param>
        /// <returns></returns>
        public static bool IsIn(this string source, params string[] sources)
        {
            if (sources == null || sources.Length == 0)
                return false;
            return sources.Contains(source);
        }

        /// <summary>
        /// 参数数组里面是否存在 source,例如: 100.IsIn(1, 5,100)= true
        /// </summary>
        /// <param name="source"></param>
        /// <param name="sources"></param>
        /// <returns></returns>
        public static bool IsIn(this int source, params int[] sources)
        {
            if (sources == null || sources.Length == 0)
                return false;
            return sources.Contains(source);
        }

        /// <summary>
        /// 参数数组里面是否存在 source,例如: 100.IsIn(1, 5,100)= true
        /// </summary>
        /// <param name="source"></param>
        /// <param name="sources"></param>
        /// <returns></returns>
        public static bool IsIn(this short source, params short[] sources)
        {
            if (sources == null || sources.Length == 0)
                return false;
            return sources.Contains(source);
        }

        /// <summary>
        /// 参数数组里面是否存在 source,例如: 100.IsIn(1, 5,100)= true
        /// </summary>
        /// <param name="source"></param>
        /// <param name="sources"></param>
        /// <returns></returns>
        public static bool IsIn(this long source, params long[] sources)
        {
            if (sources == null || sources.Length == 0)
                return false;
            return sources.Contains(source);
        }

        /// <summary>
        /// 参数数组里面是否存在 source,例如: 100.00.IsIn(1.00, 5.00,100.00)= true
        /// </summary>
        /// <param name="source"></param>
        /// <param name="sources"></param>
        /// <returns></returns>
        public static bool IsIn(this float source, params float[] sources)
        {
            if (sources == null || sources.Length == 0)
                return false;
            return sources.Contains(source);
        }

        /// <summary>
        /// 参数数组里面是否存在 source,例如: 100.00.IsIn(1.00, 5.00,100.00)= true
        /// </summary>
        /// <param name="source"></param>
        /// <param name="sources"></param>
        /// <returns></returns>
        public static bool IsIn(this double source, params double[] sources)
        {
            if (sources == null || sources.Length == 0)
                return false;
            return sources.Contains(source);
        }

        /// <summary>
        /// 参数数组里面是否存在 source,例如: 100.00.IsIn(1.00, 5.00,100.00)= true
        /// </summary>
        /// <param name="source"></param>
        /// <param name="sources"></param>
        /// <returns></returns>
        public static bool IsIn(this decimal source, params decimal[] sources)
        {
            if (sources == null || sources.Length == 0)
                return false;
            return sources.Contains(source);
        }

        /// <summary>
        /// 参数数组里面是否存在 source,例如: ASCIISortType.Asc.IsIn(ASCIISortType.Asc, ASCIISortType.Desc)= true <see cref="ASCIISortType"/>
        /// </summary>
        /// <param name="source"></param>
        /// <param name="sources"></param>
        /// <returns></returns>
        public static bool IsIn<TEnum>(this TEnum source, params TEnum[] sources) where TEnum : Enum
        {
            if (sources == null || sources.Length == 0)
                return false;
            return sources.Contains(source);
        }

        /// <summary>
        /// 判断 source 字符串中是否有 参数数组中的值，例如 "123455".IsContains("2","4","55") = true
        /// </summary>
        /// <param name="source"></param> 
        /// <param name="sources"></param>
        /// <returns></returns>
        public static bool IsContains(this string source, params string[] sources)
        {
            if (source == null)
                return false;
            if (sources == null || sources.Length == 0)
                return false;
            foreach (var s in sources)
            {
                if (source.Contains(s))
                    return true;
            }

            return false;
        }


        /// <summary>
        /// int 转 16进制 字符串
        /// </summary>
        /// <param name="source"></param>
        /// <param name="isLowercase">默认小写</param>
        /// <returns></returns>
        public static string ToHex(this int source, bool isLowercase = true)
        {
            return source.ToString(isLowercase ? "x" : "X");
        }

        /// <summary>
        /// long 转 16进制 字符串
        /// </summary>
        /// <param name="source"></param>
        /// <param name="isLowercase">默认小写</param>
        /// <returns></returns>
        public static string ToHex(this long source, bool isLowercase = true)
        {
            return source.ToString(isLowercase ? "x" : "X");
        }

        /// <summary>
        /// 克隆一个对象
        /// </summary>
        /// <typeparam name="T">必须是可序列化对象</typeparam>
        /// <param name="source">被克隆的对象</param>
        /// <returns></returns>
        public static T Clone<T>(this T source)
        {
            if (source == null)
                return default;

            // 序列化后再反序列话输出，
            var json = JsonConvert.SerializeObject(source);
            return JsonConvert.DeserializeObject<T>(json);
        }
    }
}