﻿using System.ComponentModel;
using BPA.Component.Extensions;
using NLog;

namespace BPA.Component.LogClient.Extensions
{
    /// <summary>
    /// ILoggerExtensions
    /// </summary>
    public static class INLoggerExtensions
    {
        /// <summary>
        /// LogObjToJson
        /// </summary>
        /// <typeparam name="TObj"></typeparam>
        /// <param name="obj"></param>
        /// <returns></returns>
        internal static string? LogObjToJson<TObj>(this TObj obj)
        {
            try
            {
                if (obj == null)
                    return null;
                var json = obj.TryToJson(null);
                return json;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                return null;
            }
        }


        /// <summary>
        /// DebugJson   
        /// <code> 
        /// DebugJson("message:{userInfo}",userInfo)
        /// </code>
        /// </summary>
        /// <typeparam name="TArgument1"></typeparam> 
        /// <param name="logger"></param>
        /// <param name="message"></param>
        /// <param name="argument1"></param> 
        /// <returns></returns>
        public static ILogger DebugJson<TArgument1>(this ILogger logger, [Localizable(false)] string message,
            TArgument1 argument1)
        {
            logger.Debug(message, argument1.LogObjToJson());
            return logger;
        }

        /// <summary>
        /// DebugJson
        /// <code> 
        /// DebugJson("message:{userInfo1},{userInfo2}",userInfo1,userInfo2)
        /// </code>
        /// </summary>
        /// <typeparam name="TArgument1"></typeparam>
        /// <typeparam name="TArgument2"></typeparam> 
        /// <param name="logger"></param>
        /// <param name="message"></param>
        /// <param name="argument1"></param>
        /// <param name="argument2"></param> 
        /// <returns></returns>
        public static ILogger DebugJson<TArgument1, TArgument2>(this ILogger logger,
            [Localizable(false)] string message, TArgument1 argument1, TArgument2 argument2)
        {
            logger.Debug(message, argument1.LogObjToJson(), argument2.LogObjToJson());
            return logger;
        }

        /// <summary>
        /// DebugJson
        /// <code> 
        /// DebugJson("message:{userInfo1},{userInfo2}，{userInfo3}",userInfo1,userInfo2,userInfo3)
        /// </code>
        /// </summary>
        /// <typeparam name="TArgument1"></typeparam>
        /// <typeparam name="TArgument2"></typeparam>
        /// <typeparam name="TArgument3"></typeparam>
        /// <param name="logger"></param>
        /// <param name="message"></param>
        /// <param name="argument1"></param>
        /// <param name="argument2"></param>
        /// <param name="argument3"></param>
        /// <returns></returns>
        public static ILogger DebugJson<TArgument1, TArgument2, TArgument3>(this ILogger logger,
            [Localizable(false)] string message, TArgument1 argument1, TArgument2 argument2, TArgument3 argument3)
        {
            logger.Debug(message, argument1.LogObjToJson(), argument2.LogObjToJson(), argument3.LogObjToJson());
            return logger;
        }


        /// <summary>
        /// InfoJson   
        /// <code> 
        /// InfoJson("message:{userInfo}",userInfo)
        /// </code>
        /// </summary>
        /// <typeparam name="TArgument1"></typeparam> 
        /// <param name="logger"></param>
        /// <param name="message"></param>
        /// <param name="argument1"></param> 
        /// <returns></returns>
        public static ILogger InfoJson<TArgument1>(this ILogger logger, [Localizable(false)] string message,
            TArgument1 argument1)
        {
            logger.Info(message, argument1.LogObjToJson());
            return logger;
        }

        /// <summary>
        /// InfoJson
        /// <code> 
        /// InfoJson("message:{userInfo1},{userInfo2}",userInfo1,userInfo2)
        /// </code>
        /// </summary>
        /// <typeparam name="TArgument1"></typeparam>
        /// <typeparam name="TArgument2"></typeparam> 
        /// <param name="logger"></param>
        /// <param name="message"></param>
        /// <param name="argument1"></param>
        /// <param name="argument2"></param> 
        /// <returns></returns>
        public static ILogger InfoJson<TArgument1, TArgument2>(this ILogger logger, [Localizable(false)] string message,
            TArgument1 argument1, TArgument2 argument2)
        {
            logger.Info(message, argument1.LogObjToJson(), argument2.LogObjToJson());
            return logger;
        }

        /// <summary>
        /// InfoJson
        /// <code> 
        /// InfoJson("message:{userInfo1},{userInfo2}，{userInfo3}",userInfo1,userInfo2,userInfo3)
        /// </code>
        /// </summary>
        /// <typeparam name="TArgument1"></typeparam>
        /// <typeparam name="TArgument2"></typeparam>
        /// <typeparam name="TArgument3"></typeparam>
        /// <param name="logger"></param>
        /// <param name="message"></param>
        /// <param name="argument1"></param>
        /// <param name="argument2"></param>
        /// <param name="argument3"></param>
        /// <returns></returns>
        public static ILogger InfoJson<TArgument1, TArgument2, TArgument3>(this ILogger logger,
            [Localizable(false)] string message, TArgument1 argument1, TArgument2 argument2, TArgument3 argument3)
        {
            logger.Info(message, argument1.LogObjToJson(), argument2.LogObjToJson(), argument3.LogObjToJson());
            return logger;
        }


        /// <summary>
        /// WarnJson   
        /// <code> 
        /// WarnJson("message:{userInfo}",userInfo)
        /// </code>
        /// </summary>
        /// <typeparam name="TArgument1"></typeparam> 
        /// <param name="logger"></param>
        /// <param name="message"></param>
        /// <param name="argument1"></param> 
        /// <returns></returns>
        public static ILogger WarnJson<TArgument1>(this ILogger logger, [Localizable(false)] string message,
            TArgument1 argument1)
        {
            logger.Warn(message, argument1.LogObjToJson());
            return logger;
        }

        /// <summary>
        /// WarnJson
        /// <code> 
        /// WarnJson("message:{userInfo1},{userInfo2}",userInfo1,userInfo2)
        /// </code>
        /// </summary>
        /// <typeparam name="TArgument1"></typeparam>
        /// <typeparam name="TArgument2"></typeparam> 
        /// <param name="logger"></param>
        /// <param name="message"></param>
        /// <param name="argument1"></param>
        /// <param name="argument2"></param> 
        /// <returns></returns>
        public static ILogger WarnJson<TArgument1, TArgument2>(this ILogger logger, [Localizable(false)] string message,
            TArgument1 argument1, TArgument2 argument2)
        {
            logger.Info(message, argument1.LogObjToJson(), argument2.LogObjToJson());
            return logger;
        }

        /// <summary>
        /// WarnJson
        /// <code> 
        /// WarnJson("message:{userInfo1},{userInfo2}，{userInfo3}",userInfo1,userInfo2,userInfo3)
        /// </code>
        /// </summary>
        /// <typeparam name="TArgument1"></typeparam>
        /// <typeparam name="TArgument2"></typeparam>
        /// <typeparam name="TArgument3"></typeparam>
        /// <param name="logger"></param>
        /// <param name="message"></param>
        /// <param name="argument1"></param>
        /// <param name="argument2"></param>
        /// <param name="argument3"></param>
        /// <returns></returns>
        public static ILogger WarnJson<TArgument1, TArgument2, TArgument3>(this ILogger logger,
            [Localizable(false)] string message, TArgument1 argument1, TArgument2 argument2, TArgument3 argument3)
        {
            logger.Warn(message, argument1.LogObjToJson(), argument2.LogObjToJson(), argument3.LogObjToJson());
            return logger;
        }

        /// <summary>
        /// ErrorJson   
        /// <code> 
        /// ErrorJson(exception,"message:{userInfo}",userInfo)
        /// </code>
        /// </summary>
        /// <typeparam name="TArgument1"></typeparam> 
        /// <param name="logger"></param>
        /// <param name="exception"></param>
        /// <param name="message"></param>
        /// <param name="argument1"></param> 
        /// <returns></returns>
        public static ILogger ErrorJson<TArgument1>(this ILogger logger, Exception exception,
            [Localizable(false)] string message, TArgument1 argument1)
        {
            logger.Error(exception, message, argument1.LogObjToJson());
            return logger;
        }

        /// <summary>
        /// ErrorJson
        /// <code> 
        /// ErrorJson(exception,"message:{userInfo1},{userInfo2}",userInfo1,userInfo2)
        /// </code>
        /// </summary>
        /// <typeparam name="TArgument1"></typeparam>
        /// <typeparam name="TArgument2"></typeparam> 
        /// <param name="logger"></param>
        /// <param name="exception"></param>
        /// <param name="message"></param>
        /// <param name="argument1"></param>
        /// <param name="argument2"></param> 
        /// <returns></returns>
        public static ILogger ErrorJson<TArgument1, TArgument2>(this ILogger logger, Exception exception,
            [Localizable(false)] string message, TArgument1 argument1, TArgument2 argument2)
        {
            logger.Error(exception, message, argument1.LogObjToJson());
            return logger;
        }

        /// <summary>
        /// ErrorJson
        /// <code> 
        /// ErrorJson(exception,"message:{userInfo1},{userInfo2}，{userInfo3}",userInfo1,userInfo2,userInfo3)
        /// </code>
        /// </summary>
        /// <typeparam name="TArgument1"></typeparam>
        /// <typeparam name="TArgument2"></typeparam>
        /// <typeparam name="TArgument3"></typeparam>
        /// <param name="logger"></param>
        /// <param name="exception"></param>
        /// <param name="message"></param>
        /// <param name="argument1"></param>
        /// <param name="argument2"></param>
        /// <param name="argument3"></param>
        /// <returns></returns>
        public static ILogger ErrorJson<TArgument1, TArgument2, TArgument3>(this ILogger logger, Exception exception,
            [Localizable(false)] string message, TArgument1 argument1, TArgument2 argument2, TArgument3 argument3)
        {
            logger.Error(exception, message, argument1.LogObjToJson(), argument2.LogObjToJson(),
                argument3.LogObjToJson());
            return logger;
        }
    }
}