﻿using System.Reflection;
using MongoDB.Bson;
using MongoDB.Bson.Serialization;

namespace BPA.Component.MongoClient.Conventions
{
    /// <summary>
    /// BPABsonClassMapSerializationProvider
    /// </summary>
    public class BPABsonClassMapSerializationProvider : BsonSerializationProviderBase
    {
        /// <inheritdoc />
        public override IBsonSerializer? GetSerializer(Type type, IBsonSerializerRegistry serializerRegistry)
        {
            if (type == null)
            {
                throw new ArgumentNullException("type");
            }

            var typeInfo = type.GetTypeInfo();
            if (typeInfo.IsGenericType && typeInfo.ContainsGenericParameters)
            {
                throw new ArgumentException(
                    $"Generic type {BsonUtils.GetFriendlyTypeName(type)} has unassigned type parameters.", "type");
            }

            if ((typeInfo.IsClass || (typeInfo.IsValueType && !typeInfo.IsPrimitive)) &&
                !typeof(Array).GetTypeInfo().IsAssignableFrom(type) &&
                !typeof(Enum).GetTypeInfo().IsAssignableFrom(type))
            {
                BsonClassMap bsonClassMap = BsonClassMap.LookupClassMap(type);
                return (IBsonSerializer) Activator.CreateInstance(
                    typeof(BsonClassMapSerializer<>).MakeGenericType(type), bsonClassMap)!;
            }

            return null;
        }
    }
}