﻿using MongoDB.Bson.Serialization.Conventions;

namespace BPA.Component.MongoClient.Conventions
{
    /// <summary>
    /// BPADefaultConventionPack
    /// </summary>
    public class BPADefaultConventionPack : IConventionPack
    {
        private static readonly IConventionPack __defaultConventionPack = new BPADefaultConventionPack();

        private readonly IEnumerable<IConvention> _conventions;

        /// <summary>
        /// Gets the instance.
        /// </summary>
        public static IConventionPack Instance => __defaultConventionPack;

        /// <summary>
        /// Gets the conventions.
        /// </summary>
        public IEnumerable<IConvention> Conventions => _conventions;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:MongoDB.Bson.Serialization.Conventions.DefaultConventionPack" /> class.
        /// </summary>
        private BPADefaultConventionPack()
        {
            _conventions = new List<IConvention>
            {
                new ReadWriteMemberFinderConvention(),
                new NamedIdMemberConvention("Id", "id", "_id"),
                new NamedExtraElementsMemberConvention(new string[1]
                {
                    "ExtraElements"
                }),
                new IgnoreExtraElementsConvention(ignoreExtraElements: true),
                new OldImmutableTypeClassMapConvention(),
                new NamedParameterCreatorMapConvention(),
                new StringObjectIdIdGeneratorConvention(),
                new LookupIdGeneratorConvention()
            };
        }
    }
}