﻿using System.Reflection;
using MongoDB.Bson;
using MongoDB.Bson.Serialization;

namespace BPA.Component.MongoClient.Conventions
{

    /// <summary>
    /// BPASerializationProvider
    /// </summary>
    public class BPASerializationProvider : BsonSerializationProviderBase
    {
        /// <inheritdoc/>
        public override IBsonSerializer GetSerializer(Type type, IBsonSerializerRegistry serializerRegistry)
        {
            if (type == null)
            {
                throw new ArgumentNullException("type");
            } 
            var typeInfo = type.GetTypeInfo();
            if (typeInfo.IsGenericType && typeInfo.ContainsGenericParameters)
            {
                var message = string.Format("Generic type {0} has unassigned type parameters.", BsonUtils.GetFriendlyTypeName(type));
                throw new ArgumentException(message, "type");
            } 
            if (typeInfo.IsEnum)
            {
                return CreateGenericSerializer(typeof(EnumBsonSerializer<>), new[] { type }, serializerRegistry);
            } 
            return null;
        }         
    }
}
