﻿using MongoDB.Bson;
using MongoDB.Bson.IO;
using MongoDB.Bson.Serialization;

namespace BPA.Component.MongoClient.Conventions
{
    /// <summary>
    /// JArraySerializer
    /// </summary>
    public class DictionarySerializerSerializer : IBsonSerializer<Dictionary<string, object>>
    {

        /// <summary>
        /// Representation
        /// </summary>
        private BsonType representation { set; get; }

        /// <summary>
        /// JArray
        /// </summary>
        public Type ValueType => typeof(Dictionary<string, object>);

        /// <summary>
        /// EnumBsonSerializer
        /// </summary>
        public DictionarySerializerSerializer()
        {

        }

        /// <summary>
        /// Serialize
        /// </summary>
        /// <param name="context"></param>
        /// <param name="args"></param>
        /// <param name="keyValues"></param>
        public void Serialize(BsonSerializationContext context, BsonSerializationArgs args, Dictionary<string, object> keyValues)
        {
            var bsonWriter = context.Writer;
            if (keyValues == null)
            {
                bsonWriter.WriteNull();
                return;
            }
            bsonWriter.WriteStartDocument();
            foreach (var kv in keyValues)
            {
                bsonWriter.WriteName(kv.Key);
                if (kv.Value == null)
                    bsonWriter.WriteNull();
                else
                    BsonSerializer.Serialize(bsonWriter, kv.Value.GetType(), kv.Value);
            }
            bsonWriter.WriteEndDocument();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        public Dictionary<string, object> Deserialize(BsonDeserializationContext context, BsonDeserializationArgs args)
        {
            var reader = context.Reader;
            if (reader.CurrentBsonType == BsonType.Null)
            {
                reader.ReadNull();
                return null;
            }
            var valuePairs = new Dictionary<string, object>();
            reader.ReadStartDocument();
            var trie = new BsonTrie<long>();
            while (reader.ReadBsonType() != 0)
            {
                var elementName = reader.ReadName(Utf8NameDecoder.Instance);
                object vaule;
                if (reader.CurrentBsonType == BsonType.DateTime)
                    vaule = BsonSerializer.Deserialize<DateTime>(reader);
                else
                    vaule = BsonSerializer.Deserialize<object>(reader);
                valuePairs.Add(elementName, vaule);
            }
            reader.ReadEndDocument();
            return valuePairs;
        }

        object IBsonSerializer.Deserialize(BsonDeserializationContext context, BsonDeserializationArgs args)
        {
            return Deserialize(context, args);
        }

        /// <summary>
        /// Serialize
        /// </summary>
        /// <param name="context"></param>
        /// <param name="args"></param>
        /// <param name="value"></param>
        public void Serialize(BsonSerializationContext context, BsonSerializationArgs args, object value)
        {
            Serialize(context, args, value as Dictionary<string, object>);
        }
    }
}
