﻿using MongoDB.Bson;
using MongoDB.Bson.Serialization;
using Newtonsoft.Json.Linq;

namespace BPA.Component.MongoClient.Conventions
{
    /// <summary>
    /// JArraySerializer
    /// </summary>
    public class JArraySerializer : IBsonSerializer<JArray>
    {
        /// <summary>
        /// Representation
        /// </summary>
        private BsonType representation { set; get; }

        /// <summary>
        /// JArray
        /// </summary>
        public Type ValueType => typeof(JArray);

        /// <summary>
        /// EnumBsonSerializer
        /// </summary>
        public JArraySerializer()
        {

        }

        /// <summary>
        /// Serialize
        /// </summary>
        /// <param name="context"></param>
        /// <param name="args"></param>
        /// <param name="valueArray"></param>
        public void Serialize(BsonSerializationContext context, BsonSerializationArgs args, JArray valueArray)
        {
            var bsonWriter = context.Writer;
            if (valueArray == null)
            {
                bsonWriter.WriteNull();
                return;
            }
            bsonWriter.WriteStartArray();
            foreach (var token in valueArray)
            {
                switch (token.Type)
                {
                    case JTokenType.Integer:
                        bsonWriter.WriteInt64(token.Value<long>());
                        break;
                    case JTokenType.Float:
                        bsonWriter.WriteDouble(token.Value<double>());
                        break;
                    case JTokenType.String:
                        bsonWriter.WriteString(token.Value<string>());
                        break;
                    case JTokenType.Boolean:
                        bsonWriter.WriteBoolean(token.Value<bool>());
                        break;
                    case JTokenType.Null:
                        bsonWriter.WriteNull();
                        break;
                    case JTokenType.Array:
                        BsonSerializer.Serialize(bsonWriter, token as JArray);
                        break;
                    case JTokenType.Object:
                        BsonSerializer.Serialize(bsonWriter, token as JObject);
                        break;
                    default:
                        bsonWriter.WriteString(token.Value<string>());
                        break;
                }
            }
            bsonWriter.WriteEndArray();
        }
         
        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        public object Deserialize(BsonDeserializationContext context, BsonDeserializationArgs args)
        {
            return JArray.Parse(context.Reader.ReadString());
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <param name="args"></param>
        /// <param name="value"></param>
        public void Serialize(BsonSerializationContext context, BsonSerializationArgs args, object value)
        {
            Serialize(context, args, value as JArray);
        }

        JArray IBsonSerializer<JArray>.Deserialize(BsonDeserializationContext context, BsonDeserializationArgs args)
        {
            return JArray.Parse(context.Reader.ReadString());
        }
    }
}
