﻿using MongoDB.Bson;
using MongoDB.Bson.Serialization;
using Newtonsoft.Json.Linq;

namespace BPA.Component.MongoClient.Conventions
{
    /// <summary>
    /// JObject
    /// </summary>
    public class JObjectSerializer : IBsonSerializer<JObject>
    {
        /// <summary>
        /// Representation
        /// </summary>
        private BsonType representation { set; get; }

        /// <summary>
        /// JArray
        /// </summary>
        public Type ValueType => typeof(JObject);

        /// <summary>
        /// EnumBsonSerializer
        /// </summary>
        public JObjectSerializer()
        {

        }


        /// <summary>
        /// Serialize
        /// </summary>
        /// <param name="context"></param>
        /// <param name="args"></param>
        /// <param name="jObj"></param>
        public void Serialize(BsonSerializationContext context, BsonSerializationArgs args, JObject jObj)
        {
            var bsonWriter = context.Writer;
            if (jObj == null)
            {
                bsonWriter.WriteNull();
                return;
            }
            bsonWriter.WriteStartDocument();
            foreach (var token in jObj)
            {
                //bsonWriter.WriteName(GetElementName(token.Key));
                bsonWriter.WriteName(token.Key);
                switch (token.Value.Type)
                {
                    case JTokenType.Integer:
                        bsonWriter.WriteInt64(token.Value.Value<long>());
                        break;
                    case JTokenType.Float:
                        bsonWriter.WriteDouble(token.Value.Value<double>());
                        break;
                    case JTokenType.String:
                        bsonWriter.WriteString(token.Value.Value<string>());
                        break;
                    case JTokenType.Boolean:
                        bsonWriter.WriteBoolean(token.Value.Value<bool>());
                        break;
                    case JTokenType.Null:
                        bsonWriter.WriteNull();
                        break;
                    case JTokenType.Array:
                        BsonSerializer.Serialize(bsonWriter, token.Value as JArray);
                        break;
                    case JTokenType.Object:
                        BsonSerializer.Serialize(bsonWriter, token.Value as JObject);
                        break;
                    default:
                        bsonWriter.WriteString(token.Value.Value<string>());
                        break;
                }
            }
            bsonWriter.WriteEndDocument();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        public object Deserialize(BsonDeserializationContext context, BsonDeserializationArgs args)
        {
            return JObject.Parse(context.Reader.ReadString());
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <param name="args"></param>
        /// <param name="value"></param>
        public void Serialize(BsonSerializationContext context, BsonSerializationArgs args, object value)
        {
            Serialize(context, args, value as JObject);
        }

        JObject IBsonSerializer<JObject>.Deserialize(BsonDeserializationContext context, BsonDeserializationArgs args)
        {
            return JObject.Parse(context.Reader.ReadString());
        }

        // private methods
        private string GetElementName(string memberName)
        {
            if (memberName.Length == 0)
            {
                return "";
            }
            else if (memberName.Length == 1)
            {
                return Char.ToLowerInvariant(memberName[0]).ToString();
            }
            else
            {
                return Char.ToLowerInvariant(memberName[0]) + memberName.Substring(1);
            }
        }
    }
}
