﻿using System.Linq.Expressions;
using MongoDB.Driver;
using MongoDB.Driver.Linq;

namespace BPA.Component.MongoClient.Repository;

/// <summary>
/// MongoDB仓储接口
/// </summary>
public interface IBaseMongoDbRepository<TDocument> where TDocument : new()
{
    /// <summary>
    /// 插入一个
    /// </summary>
    /// <param name="document"></param>
    /// <typeparam name="TDocument"></typeparam>
    /// <returns></returns>
    int Add<TDocument>(TDocument document);

    /// <summary>
    /// 插入多个
    /// </summary>
    /// <typeparam name="TDocument"></typeparam>
    /// <returns></returns>
    int AddList<TDocument>(IEnumerable<TDocument> documents);

    /// <summary>
    /// 修改一个
    /// </summary>
    /// <param name="document"></param>
    /// <param name="whereExpression"></param>
    /// <param name="options"></param>
    /// <returns></returns>
    /// <exception cref="ArgumentNullException"></exception>
    UpdateResult UpdateOne<TDocument>(TDocument document, Expression<Func<TDocument, bool>> whereExpression, UpdateOptions options = null);

    /// <summary>
    /// 修改多个
    /// </summary>
    /// <param name="document"></param>
    /// <param name="whereExpression"></param>
    /// <param name="options"></param>
    /// <typeparam name="TDocument"></typeparam>
    /// <returns></returns>
    UpdateResult UpdateMany<TDocument>(TDocument document, Expression<Func<TDocument, bool>> whereExpression, UpdateOptions options = null);

    /// <summary>
    /// 删除一个
    /// </summary>
    /// <param name="document"></param>
    /// <param name="whereExpression"></param>
    /// <typeparam name="TDocument"></typeparam>
    /// <returns></returns>
    /// <exception cref="ArgumentNullException"></exception>
    DeleteResult DeleteOne<TDocument>(TDocument document, Expression<Func<TDocument, bool>> whereExpression);

    /// <summary>
    /// 删除多个
    /// </summary>
    /// <param name="document"></param>
    /// <param name="whereExpression"></param>
    /// <typeparam name="TDocument"></typeparam>
    /// <returns></returns>
    /// <exception cref="ArgumentNullException"></exception>
    DeleteResult DeleteMany<TDocument>(TDocument document, Expression<Func<TDocument, bool>> whereExpression);

    /// <summary>
    /// 查询扩展
    /// </summary>
    /// <param name="document"></param>
    /// <param name="whereExpression"></param>
    /// <typeparam name="TDocument"></typeparam>
    /// <returns></returns>
    IMongoQueryable<TDocument> Queryable<TDocument>(TDocument document, Expression<Func<TDocument, bool>> whereExpression);
}