﻿using BPA.Component.ApolloClient;
using BPA.Component.DbClientTester;
using BPA.Component.MongoClientTester.Models;
using BPA.Component.MongoClientTester.Repositorys;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using MongoDB.Bson;

namespace BPA.Component.MongoClientTester;

internal class Program
{
    private static async Task Main()
    {
        var builder = new ConfigurationBuilder();
        builder.AddEnvironmentVariables();
        IConfiguration configuration = builder.Build();
        configuration.AddApolloConfiguration();

        var services = new ServiceCollection();
        services.AddApollo<TestConfig>();
        services.AddLogging(configure => { configure.AddConsole(); });
        services.AddScoped<TestMongoDcClient>();
        services.AddTransient<ICorpInfoRepository, CorpInfoRepository>();

        var serviceProvider = services.BuildServiceProvider();
        var userRepository = serviceProvider.GetService<ICorpInfoRepository>();

        var data = new CorpInfo
        {
            Id = DateTime.Now.Ticks,
            DB = long.MaxValue,
            AddressInfos = new List<CorpAddressInfo>
            {
                new() {Id = DateTime.Now.Ticks, Latitude = 66.52, Longitude = 451.63, Address = "天府大道天府三街", Remark = "一号办公地点1"},
                new() {Id = DateTime.Now.Ticks, Latitude = 66.52, Longitude = 451.63, Address = "天府大道天府五街", Remark = "二号办公地点1"},
            },
            Name = "太阳国",
            No = "sun",
            Creator = new CorpAddressInfo {Id = DateTime.Now.Ticks, Latitude = 66.52, Longitude = 451.63, Address = "天府大道天府三街", Remark = "一号办公地点"},
            Tags = new Dictionary<string, object>
            {
                {"datetime", DateTime.Now},
                {"array0-1", new object[] {1, true, 66.2D, Enums.Aaa, DateTime.Now, ""}},
                {"array00", null},
                {"array01", new List<string> {"sss", "bbb"}},
                {"array02", new List<string[]> {new[] {"1", "2"}, new[] {"3", "4"}}},
                {"array03", new List<CorpAddressInfo> {new() {Address = "ss", Id = DateTime.Now.Ticks, Latitude = 1, Longitude = 2, Remark = ""}}},
            },
            Phones = new List<string> {"1921682012", "1921682012", "1921682012"}
        };

        // 添加一个
        // userRepository.Add(data);
        // // 添加多个
        // userRepository.AddList(new List<CorpInfo>
        // {
        //     new()
        //     {
        //         Id = DateTime.Now.Ticks,
        //         DB = long.MaxValue,
        //         AddressInfos = new List<CorpAddressInfo>
        //         {
        //             new() {Id = DateTime.Now.Ticks, Latitude = 66.52, Longitude = 451.63, Address = "天府大道天府三街", Remark = "一号办公地点"},
        //             new() {Id = DateTime.Now.Ticks, Latitude = 66.52, Longitude = 451.63, Address = "天府大道天府五街", Remark = "二号办公地点"},
        //         },
        //         Name = "太阳国1",
        //         No = "sun1",
        //         Creator = new CorpAddressInfo {Id = DateTime.Now.Ticks, Latitude = 66.52, Longitude = 451.63, Address = "天府大道天府三街", Remark = "一号办公地点"},
        //         Tags = new Dictionary<string, object>
        //         {
        //             {"datetime", DateTime.Now},
        //             {"array0-1", new object[] {1, true, 66.2D, Enums.Aaa, DateTime.Now, ""}},
        //             {"array00", null},
        //             {"array01", new List<string> {"sss", "bbb"}},
        //             {"array02", new List<string[]> {new[] {"1", "2"}, new[] {"3", "4"}}},
        //             {"array03", new List<CorpAddressInfo> {new() {Address = "ss", Id = DateTime.Now.Ticks, Latitude = 1, Longitude = 2, Remark = ""}}},
        //         },
        //         Phones = new List<string> {"1921682012", "1921682012", "1921682012"}
        //     },
        //     new()
        //     {
        //         Id = DateTime.Now.Ticks,
        //         DB = long.MaxValue,
        //         AddressInfos = new List<CorpAddressInfo>
        //         {
        //             new() {Id = DateTime.Now.Ticks, Latitude = 66.52, Longitude = 451.63, Address = "天府大道天府三街", Remark = "一号办公地点"},
        //             new() {Id = DateTime.Now.Ticks, Latitude = 66.52, Longitude = 451.63, Address = "天府大道天府五街", Remark = "二号办公地点"},
        //         },
        //         Name = "太阳国1",
        //         No = "sun1",
        //         Creator = new CorpAddressInfo {Id = DateTime.Now.Ticks, Latitude = 66.52, Longitude = 451.63, Address = "天府大道天府三街", Remark = "一号办公地点"},
        //         Tags = new Dictionary<string, object>
        //         {
        //             {"datetime", DateTime.Now},
        //             {"array0-1", new object[] {1, true, 66.2D, Enums.Aaa, DateTime.Now, ""}},
        //             {"array00", null},
        //             {"array01", new List<string> {"sss", "bbb"}},
        //             {"array02", new List<string[]> {new[] {"1", "2"}, new[] {"3", "4"}}},
        //             {"array03", new List<CorpAddressInfo> {new() {Address = "ss", Id = DateTime.Now.Ticks, Latitude = 1, Longitude = 2, Remark = ""}}},
        //         },
        //         Phones = new List<string> {"1921682012", "1921682012", "1921682012"}
        //     }
        // });
        // // 更新一个
        // userRepository.UpdateOne(data, info => info.Id == 637843434739905640);
        // // 更新多个
        // userRepository.UpdateMany(data, info => info.Id == 637843434739905640);
        // 删除一个
        // userRepository.DeleteOne(data, info => info.Id == 637843434739905640);
        // 删除多个
        // userRepository.DeleteMany(data, info => info.Id == 637843460358762663);
        // 查询
        var dd = userRepository.Queryable(data, info => info.Id == 0).ToList();
        Console.WriteLine(dd.ToJson());
    }
}

public enum Enums
{
    Aaa = 1,
    Bbb = 2
}