﻿using BPA.Component.RabbitMQClient.Options;
using BPA.Component.RabbitMQClient.Provider;
using RabbitMQ.Client;
using ExchangeType = BPA.Component.RabbitMQClient.Enums.ExchangeType;

namespace BPA.Component.RabbitMQClient.Config
{
    /// <summary>
    /// 交换机配置
    /// </summary>
    public class RabbitMqExchangeConfig
    {

        /// <summary>
        /// 交换机配置参数
        /// </summary>
        public ExchangeOption Option { protected set; get; }

        /// <summary>
        /// channel
        /// </summary>
        internal IModel Channel { get; private set; }

        /// <summary>
        /// RabbitMQProvider
        /// </summary>
        internal RabbitMQProvider RabbitMQProvider { get; }


        /// <summary>
        /// 构造方法
        /// </summary>
        /// <param name="rabbitMQProvider"></param>
        /// <param name="option"></param>  
        public RabbitMqExchangeConfig(RabbitMQProvider rabbitMQProvider, ExchangeOption option)
        {
            RabbitMQProvider = rabbitMQProvider;
            this.Option = option;
            this.RabbitMQProvider.OnRecoveryConnect += InitExchange;
        }

        /// <summary>
        /// 初始化交换机
        /// </summary>
        internal void InitExchange()
        {
            //创建通道
            Channel = RabbitMQProvider.Connection.CreateModel();
            //声明交换机
            //配合channel(durable = true),queue(durable)可以实现，即使服务器宕机，消息仍然保留 
            if (Option.Type == ExchangeType.X_Delayed_Message)
            {
                Option.Arguments.Add("x-delayed-type", "direct");
            }
            Channel.ExchangeDeclare(Option.Name, Option.Type.ToString().ToLower().Replace("_", "-"), Option.Durable, Option.AutoDelete, Option.Arguments);
            Console.WriteLine();
            Console.WriteLine($"*bpa.rabbitmqclient* 初始化交换机: start ---------------------------");
            Console.WriteLine($"*bpa.rabbitmqclient*  [Exchange]={ Option.Name.Name}");
            Console.WriteLine($"*bpa.rabbitmqclient*  [FullName]={ this.GetType().FullName}");
            Console.WriteLine($"*bpa.rabbitmqclient*  [HashCode]={ this.GetHashCode()}");
            Console.WriteLine($"*bpa.rabbitmqclient*  [SessionID]={ Channel}");
            Console.WriteLine($"*bpa.rabbitmqclient* 初始化交换机: end ---------------------------");
        }
    }
}
