﻿using BPA.Component.RabbitMQClient.Options;
using BPA.Component.RabbitMQClient.Provider;
using RabbitMQ.Client;

namespace BPA.Component.RabbitMQClient.Config
{
    /// <summary>
    /// 队列配置
    /// </summary>
    public class RabbitMqQueueConfig
    {
        /// <summary>
        /// 队列配置参数
        /// </summary>
        public QueueOption Option { protected set; get; }

        /// <summary>
        /// channel
        /// </summary>
        internal IModel Channel { get; private set; }

        /// <summary>
        /// RabbitMQProvider
        /// </summary>
        internal RabbitMQProvider RabbitMQProvider { get; }

        /// <summary>
        /// 初始化消费者的方法
        /// </summary>
        internal Action InitConsumer { set; get; }

        /// <summary>
        /// 消息队列配置参数
        /// </summary>
        /// <param name="rabbitMQProvider">rabbitMQProvider</param>
        /// <param name="option"></param>
        public RabbitMqQueueConfig(RabbitMQProvider rabbitMQProvider, QueueOption option)
        {
            RabbitMQProvider = rabbitMQProvider;
            Option = option;
            RabbitMQProvider.OnRecoveryConnect += ReStart;
        }

        /// <summary>
        /// 初始化队列
        /// </summary>
        internal void InitQueue()
        {
            //创建一个通道
            Channel = RabbitMQProvider.Connection.CreateModel();
            if (Option.QueueName.IsDynamic)
            {
                //动态队列不需要持久化队列
                Option.AutoDelete = true;
                Option.Durable = false;
            }

            //声明队列 
            Channel.QueueDeclare(queue: Option.QueueName,
                                   durable: Option.Durable,
                                   exclusive: Option.Exclusive,
                                   autoDelete: Option.AutoDelete,
                                   arguments: Option.Arguments); 
            //队列绑定到交换机  交换机名字存在才绑定
            if (!string.IsNullOrEmpty(Option.ExchangeName))
                Channel.QueueBind(Option.QueueName, Option.ExchangeName, Option.RoutingKey, Option.Arguments);

            //consumer.Received -= registerConsumer.OnReceived;
            //autoAck：是否自动ack，如果不自动ack，需要使用channel.ack、channel.nack、channel.basicReject 进行消息应答
            Channel.BasicQos(0, 1, false);
            Console.WriteLine();
            Console.WriteLine($"*bpa.rabbitmqclient* 初始化队列: start ---------------------------");
            Console.WriteLine($"*bpa.rabbitmqclient* [{nameof(Option.QueueName)}]={ Option.QueueName}");
            Console.WriteLine($"*bpa.rabbitmqclient* [FullName]={ this.GetType().FullName}");
            Console.WriteLine($"*bpa.rabbitmqclient* [HashCode]={ this.GetHashCode()}");
            Console.WriteLine($"*bpa.rabbitmqclient* [Exchange]={Option.ExchangeName}");
            Console.WriteLine($"*bpa.rabbitmqclient* 初始化队列: end ---------------------------");
        }

        /// <summary>
        /// 重新连接
        /// </summary>
        internal void ReStart()
        {
            InitQueue();
            InitConsumer?.Invoke();
        }
    }


}
