﻿using BPA.Component.RabbitMQClient.Base;
using BPA.Component.RabbitMQClient.Config;
using BPA.Component.RabbitMQClient.Options;
using BPA.Component.RabbitMQClient.Result;

namespace BPA.Component.RabbitMQClient.Extensions
{
    /// <summary>
    /// 队列配置扩展
    /// </summary>
    public static class RabbitMqExchangeConfigExtension
    {

        /// <summary>
        /// 配置交换机选项
        /// </summary>
        /// <param name="config"></param>
        /// <param name="configAction"></param>
        /// <returns></returns>
        public static RabbitMqExchangeConfig ConfigOption(this RabbitMqExchangeConfig config, Action<ExchangeOption> configAction)
        {
            if (configAction == null)
                return config;
            configAction.Invoke(config.Option);
            return config;
        }

        /// <summary>
        /// 使用交换机
        /// </summary>
        /// <param name="config"></param> 
        /// <returns></returns>
        public static RabbitMqExchangeConfig UseExchange(this RabbitMqExchangeConfig config)
        {
            config.InitExchange();
            return config;
        }

        /// <summary>
        /// 启用生产者,交换机模式；消息-->交换机-->消息队列-->消费者
        /// </summary>
        /// <typeparam name="TProducer"></typeparam>
        /// <typeparam name="TMsgBody"></typeparam>
        /// <param name="config"></param> 
        /// <returns></returns>
        public static IRegisterProducerResult UseProducer<TMsgBody, TProducer>(this RabbitMqExchangeConfig config) where TProducer : BaseProducer<TMsgBody, TProducer>
        {
            if (config == null)
                throw new ArgumentNullException(nameof(config));
            var producer = config.RabbitMQProvider.ServiceProvider.GetService(typeof(TProducer)) as BaseProducer<TMsgBody, TProducer>;
            if (producer == null)
                throw new ArgumentNullException("请检查您的生产者是否注入的IOC容器,请调用 UseRabbitMQProducer 方法进行注入");
            if (producer.Sender != null)
                throw new ArgumentException($"当前生产者[{producer.GetType().FullName}]已经注册过了，不能再注册了");
            producer.Init(config);
            return new RegisterProducerResult<TMsgBody, TProducer>(producer);
        }
    }
}
