﻿using BPA.Component.RabbitMQClient.Enums;

namespace BPA.Component.RabbitMQClient.Options
{
    /// <summary>
    /// 交换机配置
    /// </summary>
    public class ExchangeOption
    {
        /// <summary>
        /// 交换机名称
        /// </summary>
        public ExchangeName Name { set; get; }

        /// <summary>
        /// 交换机类型,默认Direct
        /// type：有direct、fanout、topic , headers  四种
        /// </summary>
        public ExchangeType Type { get; set; } = ExchangeType.Direct;

        private string routingKey = string.Empty;
        /// <summary>
        /// 路由key,默认为string.Empty
        /// </summary>
        public string RoutingKey { set => routingKey = value ?? string.Empty; get => routingKey; }

        /// <summary>
        /// 是否持久化交换机，默认true
        /// durable：true、false true：服务器重启会保留下来Exchange。
        /// 警告：仅设置此选项，不代表消息持久化。即不保证重启后消息还在。
        /// 原文：true if we are declaring a durable exchange (the exchange will survive a server restart)
        /// </summary>
        public bool Durable { set; get; } = true;

        /// <summary>
        /// 自动删除，默认false
        /// autoDelete:true、false.true:当已经没有消费者时，
        /// 服务器是否可以删除该Exchange。
        /// 原文1：true if the server should delete the exchange when it is no longer in use。
        /// </summary>
        public bool AutoDelete { set; get; } = false;

        /// <summary>
        /// 消息投递模式 ，默认持久化<see cref="DeliveryModeType.Persistent"/>,  是否持久化消息, 
        /// </summary>
        public DeliveryModeType DeliveryMode { set; get; } = DeliveryModeType.Persistent;

        /// <summary>
        /// 扩展参数
        /// <code>
        ///    <see cref="ExchangeType.ExchangeType"/>:
        ///    x-match = all ：表示所有的键值对都匹配才能接受到消息;
        ///    x-match = any ：表示只要有键值对匹配就能接受到消息; 
        ///     ---------
        /// 
        ///  </code>
        /// </summary>
        public IDictionary<string, object> Arguments { set; get; } = new Dictionary<string, object>();

    }
}
