﻿namespace BPA.Component.RabbitMQClient.Options
{
    /// <summary>
    /// 队列名称
    /// </summary>
    public class QueueName
    {

        /// <summary>
        /// 构建一个队列的名字
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public static QueueName Build(string name) => new QueueName { Name = name };

        /// <summary>
        /// 构建一个动态的名字
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public static QueueName BuildDynamic(string name) => new QueueName { Name = $"{ name}_{Guid.NewGuid():n}", IsDynamic = true };

        /// <summary>
        /// 队列名称
        /// </summary>
        public string Name { private set; get; }

        /// <summary>
        /// 是否动态, 动态队列不需要持久化队列
        /// </summary>
        public bool IsDynamic { private set; get; }

        /// <summary>
        /// 隐式 转换为string
        /// </summary>
        /// <param name="queueName"></param>
        public static implicit operator string(QueueName queueName) => queueName.Name;

        /// <summary>
        /// ToString
        /// </summary>
        /// <returns></returns>
        public override string ToString() => Name;
    }
}
