﻿using System.Diagnostics;
using BPA.Component.RabbitMQClient.Base;

namespace BPA.Component.RabbitMQClient.Result
{
    /// <summary>
    /// RegisterConsumerResult
    /// </summary>
    public class RegisterConsumerResult<TMsgBody, TConsumer> : IRegisterConsumerResult
    {
        private BaseConsumer<TMsgBody, TConsumer> registerConsumer;

        /// <summary>
        /// RegisterResult
        /// </summary>
        /// <param name="registerConsumer"></param> 
        public RegisterConsumerResult(BaseConsumer<TMsgBody, TConsumer> registerConsumer)
        {
            this.registerConsumer = registerConsumer;
        }


        /// <summary>
        /// Dispose
        /// </summary>
        public void Dispose()
        {
            registerConsumer.Dispose();
            registerConsumer = null;
        }

        /// <summary>
        /// 暂停接收消息
        /// </summary>
        public void Pause()
        {
            lock (registerConsumer)
            {
                registerConsumer.ColseChannel();
                var msg = $"RegisterConsumer:[Colse]==>" +
                          $"\tQueueName:[{registerConsumer.Config.Option.QueueName}],\n" +
                          $"\tFullName:[{registerConsumer.GetType().FullName}],\n" +
                          $"\tHashCode:[{registerConsumer.GetHashCode()}],\n" +
                          $"\tConsumerID:[{registerConsumer.ConsumeId}]\n";
                Trace.WriteLine(msg);
                Console.WriteLine(msg);
            }
        }

        /// <summary>
        /// 重新开始接受消息
        /// </summary>
        public void ReStart()
        {
            lock (registerConsumer)
            {
                var isReStart = false;
                if (registerConsumer.ChannelIsColsed)
                {
                    registerConsumer.ReStart();
                    isReStart = true;
                }
                var msg = $"RegisterConsumer:[ReStart:{isReStart}]==>" +
                            $"\tQueueName:[{registerConsumer.Config.Option.QueueName}],\n" +
                            $"\tFullName:[{registerConsumer.GetType().FullName}],\n" +
                            $"\tHashCode:[{registerConsumer.GetHashCode()}],\n" +
                            $"\tConsumerID:[{registerConsumer.ConsumeId}]\n";
                Trace.WriteLine(msg);
                Console.WriteLine(msg);
            }
        }
    }
}
