﻿namespace BPA.Component.RedisClient;

/// <summary>
/// 全局 rediskey
/// </summary>
public class GlobalRedisKey
{
    /// <summary>
    /// 前缀
    /// </summary>
    public const string Prefix = "Global";

    #region MongoDB业务数据库Key

    /// <summary>
    /// 企业的 [业务日志] MongoDB数据库Key
    /// </summary>
    /// <param name="corpId"></param>
    /// <returns></returns>
    public static string CorpMongoBizLog(long corpId) => BuildRedisKey("CorpMongo", "BizLog", corpId);

    /// <summary>
    /// 企业的 [消息] MongoDB数据库Key
    /// </summary>
    /// <param name="corpId"></param>
    /// <returns></returns>
    public static string CorpMongoMessage(long corpId) => BuildRedisKey("CorpMongo", "Message", corpId);

    /// <summary>
    /// 企业的 [附件] MongoDB数据库Key
    /// </summary>
    /// <param name="corpId"></param>
    /// <returns></returns>
    public static string CorpMongoAnnex(long corpId) => BuildRedisKey("CorpMongo", "Annex", corpId);

    #endregion

    /// <summary>
    /// 创建 rediskey
    /// </summary>
    /// <param name="keys"></param>
    /// <returns></returns>
    private static string BuildRedisKey(params object[] keys)
    {
        if (keys == null || keys.Length == 0)
            throw new ArgumentNullException($"参数{keys},不能为空或者长度为0");
        return $"{Prefix}:{string.Join(":", keys)}";
    }
}