﻿using BPA.Component.ApolloClient;
using BPA.Component.ApolloClientTester;
using BPA.Component.Extensions;
using BPA.Component.RedisClient;
using CSRedis;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;

namespace BPA.Component.RedisClientTester
{
    class Program
    {
        static void Main(string[] args)
        {
            // Console.WriteLine("1=测试单例,2=测试工厂");
            // var cmd = Console.ReadLine();
            // if (cmd == "1")
            UseRedisSingleton();
            // else if (cmd == "2")
            //     TestUseRedisFactory();
            // else
            //     Console.WriteLine("命令错误");
            Console.ReadLine();
        }

        /// <summary>
        /// 测试单例
        /// </summary>
        static void UseRedisSingleton()
        {
            var builder = new ConfigurationBuilder();
            builder.AddEnvironmentVariables();
            var configuration = builder.Build();
            configuration.AddApolloConfiguration();
            var services = new ServiceCollection();
            services.AddApollo<TestConfig>();

            services.UseRedisSingleton(sp => GetRedisSingletonConnectionString(sp.GetService<TestConfig>() ?? throw new InvalidOperationException()));

            var serviceProvider = services.BuildServiceProvider();
            var redisClient = serviceProvider.GetService<CSRedisClient>();

            // 扫描
            var employeeLoginAuthKey = "testfKeys:*";
            var scanKeys = redisClient.Keys(employeeLoginAuthKey);
            scanKeys.Each(key => redisClient.Del(key));

            var redisKey = "testfKeys";
            var fKeys = new string[] {"1", "2", "3", "4"};
            (string key, string value)[] kvs = redisClient.CacheShell(redisKey, fKeys, -1, keys =>
            {
                var result = new (string, string)[keys.Length];
                for (var i = 0; i < keys.Length; i++)
                {
                    var item = keys[i];
                    result[i] = (item, null);
                }

                // return result;
                return result;
            });
        }

        /// <summary>
        /// 测试redis 工厂
        /// </summary>
        static void UseRedisFactory()
        {
            var builder = new ConfigurationBuilder();
            builder.AddEnvironmentVariables();
            var configuration = builder.Build();
            configuration.AddApolloConfiguration();

            var services = new ServiceCollection();
            services.AddApollo<TestConfig>();
            services.UseRedisFactory<TestRedisClientFactory>();

            var serviceProvider = services.BuildServiceProvider();
            var redisClientFactory = serviceProvider.GetService<TestRedisClientFactory>();

            TestRedisClient(redisClientFactory.CSRedisClient01, "s01");
            TestRedisClient(redisClientFactory.CSRedisClient02, "s01");
            TestRedisClient(redisClientFactory.CSRedisClient03, "s01");

            //监听配置变化后 在进行测试
            redisClientFactory.OnRedisChange = () =>
            {
                TestRedisClient(redisClientFactory.CSRedisClient01, "s02");
                TestRedisClient(redisClientFactory.CSRedisClient02, "s02");
                TestRedisClient(redisClientFactory.CSRedisClient03, "s02");
            };
        }


        /// <summary>
        /// 获取单例redis 连接字符串
        /// </summary>
        /// <param name="config"></param>
        /// <returns></returns>
        static string GetRedisSingletonConnectionString(TestConfig config)
        {
            var conStr = config.RedisConfig.First();
            if (conStr.IndexOf("prefix") <= 0)
                conStr += "";
            return conStr;
        }

        /// <summary>
        /// 测试redis client 操作redis 集群
        /// </summary>
        /// <param name="redisClient"></param>
        static void TestRedisClient(CSRedisClient redisClient, string prefix)
        {
            var key = $"{prefix}:testredisclient";
            redisClient.Set(key, $"{DateTime.Now}-{Guid.NewGuid()}", 300);
            Console.WriteLine(redisClient.Get(key));
            Thread.Sleep(2000);
        }
    }
}