﻿using System.ComponentModel;
using BPA.Component.RedisClient;
using BPA.Component.RedisClientTester;
using Com.Ctrip.Framework.Apollo.Enums;

namespace BPA.Component.ApolloClientTester
{
    /// <summary>
    /// redis 工厂, 假设连接了三个 
    /// </summary>
    public class TestRedisClientFactory : RedisClientFactory
    {
        /// <summary>
        /// redis 集群1
        /// </summary>
        public CSRedis.CSRedisClient CSRedisClient01 { private set; get; }

        /// <summary>
        /// redis 集群2
        /// </summary>
        public CSRedis.CSRedisClient CSRedisClient02 { private set; get; }

        /// <summary>
        /// redis 集群3
        /// </summary>
        public CSRedis.CSRedisClient CSRedisClient03 { private set; get; }

        /// <summary>
        /// testConfig
        /// </summary>
        private readonly TestConfig testConfig;

        /// <summary>
        /// 测试委托，方便配置改变后触发
        /// </summary>
        public Action OnRedisChange;

        /// <summary>
        /// TestRedisClientFactory
        /// </summary>
        /// <param name="testConfig"></param>
        public TestRedisClientFactory(TestConfig testConfig)
        {
            this.testConfig = testConfig;
            CreateRedisClients(testConfig);
            this.testConfig.OnConfigChange += TestConfig_OnConfigChange;
        }

        /// <summary>
        /// 监听 redis config 改变
        /// </summary>
        /// <param name="config"></param>
        /// <param name="key"></param>
        /// <param name="newValue"></param>
        /// <param name="oldValue"></param>
        /// <param name="changeType"></param>
        private void TestConfig_OnConfigChange(TestConfig config, string key, string newValue, string oldValue,
            PropertyChangeType changeType)
        {
            if (!Enum.IsDefined(typeof(PropertyChangeType), changeType))
                throw new InvalidEnumArgumentException(nameof(changeType), (int) changeType, typeof(PropertyChangeType));

            //这里方法上的 config 和 上面定义的 testConfig 是一样的，因为注入的配置对象是单例的
            if (nameof(config.RedisConfig) != key) return;

            //真实情况下，多个集群应该是多个配置，这里简单处理下
            CreateRedisClients(config);

            OnRedisChange?.Invoke();
        }

        /// <summary>
        /// 模拟创建了三个对象
        /// </summary>
        /// <param name="config"></param>
        private void CreateRedisClients(TestConfig config)
        {
            CSRedisClient01 = CreateCsRedisClient(ProccessRedisConnectionString(config.RedisConfig[0], 1));
            CSRedisClient02 = CreateCsRedisClient(ProccessRedisConnectionString(config.RedisConfig[1], 2));
            CSRedisClient03 = CreateCsRedisClient(ProccessRedisConnectionString(config.RedisConfig[2], 3));
        }

        /// <summary>
        /// 处理 redis 连接字符串
        /// </summary>
        /// <param name="connectionString"></param>
        /// <param name="index"></param>
        /// <returns></returns>
        private static string ProccessRedisConnectionString(string connectionString, int index)
        {
            if (connectionString.IndexOf("prefix", StringComparison.Ordinal) <= 0)
                connectionString += $",prefix=testfactory{index}:";
            return connectionString;
        }
    }
}