﻿using System.Text;
using BPA.Component.DTOCommon.JsonConverters;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Newtonsoft.Json.Serialization;
using Refit;

namespace BPA.Component.SDKCommon;

/// <summary>
/// BPANewtonsoftJsonContentSerializer
/// </summary>
// ReSharper disable once InconsistentNaming
public class BPANewtonsoftJsonContentSerializer : IContentSerializer
{
    /// <summary>
    /// logger
    /// </summary>
    private readonly ILogger<BPANewtonsoftJsonContentSerializer> _logger;

    /// <summary>
    /// The <see cref="Lazy{T}"/> instance providing the JSON serialization settings to use
    /// </summary>                                                                                           
    internal static readonly JsonSerializerSettings JsonSerializerSettings;

    /// <summary>
    /// BPANewtonsoftJsonContentSerializer
    /// </summary>
    static BPANewtonsoftJsonContentSerializer()
    {
        JsonSerializerSettings = new JsonSerializerSettings
        {
            //设置json日期格式
            //options.SerializerSettings.Converters.Add(new IsoDateTimeConverter { DateTimeFormat = "yyyy-MM-dd HH:mm:ss:ffff" });
            //options.SerializerSettings.DateFormatString = "yyyy'-'MM'-'dd' 'HH':'mm':'ss.FFFFFFF";
            ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
            ContractResolver = new CamelCasePropertyNamesContractResolver()
        };
        JsonSerializerSettings.Converters.Add(new JsUlongConverter());
        JsonSerializerSettings.Converters.Add(new JsDateTimeConverter());
    }

    /// <summary>
    /// Creates a new <see cref="BPANewtonsoftJsonContentSerializer"/> instance with the specified parameters
    /// </summary> 
    public BPANewtonsoftJsonContentSerializer(ILogger<BPANewtonsoftJsonContentSerializer> logger)
    {
        _logger = logger;
    }

    /// <summary>
    /// DeserializeAsync
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="content"></param>
    /// <returns></returns>
    public async Task<T?> DeserializeAsync<T>(HttpContent content)
    {
        string? json = null;
        try
        {
            json = await content.ReadAsStringAsync().ConfigureAwait(false);

            return JsonConvert.DeserializeObject<T>(json, JsonSerializerSettings);
        }
        catch (Exception ex)
        {
            if (!SDKExtensions.IsDebug)
                _logger.LogError(ex, json);
            throw;
        }
    }

    /// <summary>
    /// SerializeAsync
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="item"></param>
    /// <returns></returns>
    public Task<HttpContent> SerializeAsync<T>(T item)
    {
        var json = JsonConvert.SerializeObject(item, JsonSerializerSettings);
        var content = new StringContent(json, Encoding.UTF8, "application/json");

        return Task.FromResult((HttpContent) content);
    }
}