﻿using System;
using BPA.Component.DTOCommon.Langs;
using Microsoft.Extensions.DependencyInjection;
using Refit;

namespace BPA.Component.SDKCommon
{
    /// <summary>
    /// BaseSDK
    /// </summary>
    public abstract class BaseApiSDK<TApiSDK> where TApiSDK : BaseApiSDK<TApiSDK>
    {
        /// <summary>
        /// ServiceProvider
        /// </summary>
        protected readonly IServiceProvider ServiceProvider;

        /// <summary>
        /// HttpClientFactory
        /// </summary>
        protected readonly IHttpClientFactory? HttpClientFactory;

        /// <summary>
        /// GetApiUrlFunc
        /// </summary>
        protected readonly Func<IServiceProvider, string> GetApiUrlFunc;

        /// <summary>
        /// LangType
        /// </summary>
        protected readonly LangType LangType;

        /// <summary>
        /// 超时时间 ,默认60秒
        /// </summary>
        public int Timeout { set; get; } = 60;

        /// <summary>
        /// RefitSettings
        /// </summary>
        private readonly RefitSettings? _refitSettings;

        /// <summary>
        /// HttpClient
        /// </summary>
        private HttpClient _httpClient;

        /// <summary>
        /// BasicApiSDK
        /// </summary>
        /// <param name="serviceProvider"></param>
        /// <exception cref="ArgumentNullException"></exception>
#pragma warning disable CS8618
        public BaseApiSDK(IServiceProvider serviceProvider)
#pragma warning restore CS8618
        {
            ServiceProvider = serviceProvider;
            HttpClientFactory = serviceProvider.GetService<IHttpClientFactory>();
            _refitSettings = serviceProvider.GetService<RefitSettings>();

            var apiUrlFuncCollection = serviceProvider.GetService<SDKUrlFuncCollection>() ??
                                       throw new ArgumentNullException($"serviceProvider.GetService<SDKUrlFuncCollection>()");
            GetApiUrlFunc = apiUrlFuncCollection.Get<TApiSDK>();
            LangType = apiUrlFuncCollection.GetClientLangFunc(serviceProvider);
        }

        /// <summary>
        /// For
        /// </summary>
        /// <typeparam name="TApi"></typeparam> 
        /// <returns></returns>
        protected TApi For<TApi>()
        {
            // ReSharper disable once ConditionIsAlwaysTrueOrFalse
            if (_httpClient == null)
            {
                _httpClient = HttpClientFactory.CreateClient(
                    typeof(TApiSDK).FullName ?? throw new InvalidOperationException("TApiSDK null"));
                _httpClient.Timeout = TimeSpan.FromSeconds(Timeout);
                var baseAddress = GetApiUrlFunc(ServiceProvider);
                _httpClient.BaseAddress = new Uri(baseAddress);
                _httpClient.DefaultRequestHeaders.Add("Accept-Language", LangType.ToString().ToLower());
            }

            return RestService.For<TApi>(_httpClient, _refitSettings);
        }
    }
}