﻿using BPA.Component.DTOCommon.Langs;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.DependencyInjection;
using Refit;

namespace BPA.Component.SDKCommon
{
    /// <summary>
    /// ApiSDKExtensions
    /// </summary>
    public static class SDKExtensions
    {
        /// <summary>
        /// 是否调试
        /// </summary>
        public static bool IsDebug { set; get; }

        /// <summary>
        /// apiUrlFuncCollection
        /// </summary>
        internal readonly static SDKUrlFuncCollection ApiUrlFuncCollection = new();

        /// <summary>
        /// 初始化ApiSDK配置
        /// </summary>
        /// <param name="services"></param>
        /// <param name="getClientLangFunc"></param>
        /// <param name="isDebug"></param>
        /// <returns></returns>
        public static IServiceCollection UseApiSDK(this IServiceCollection services, Func<IServiceProvider, LangType> getClientLangFunc,
            bool isDebug = false)
        {
            IsDebug = isDebug;
            ApiUrlFuncCollection.GetClientLangFunc = getClientLangFunc;
            services.AddSingleton<IHttpContextAccessor, HttpContextAccessor>();
            services.AddSingleton(_ => ApiUrlFuncCollection);
            services.AddTransient<SDKHttpMessageHandler>();
            services.AddTransient<SDKHttpMessageHandlerWithTrackId>();
            services.AddSingleton<BPANewtonsoftJsonContentSerializer>();
            services.AddSingleton(sp => new RefitSettings {ContentSerializer = sp.GetService<BPANewtonsoftJsonContentSerializer>()});
            return services;
        }

        /// <summary>
        /// 是否调试
        /// </summary>
        /// <param name="serviceProvider"></param>
        /// <param name="isDebug"></param>
        public static void SetDebug(this IServiceProvider serviceProvider, bool isDebug)
        {
            IsDebug = isDebug;
        }

        /// <summary>
        /// UseAccountApiSDK
        /// </summary>
        /// <param name="services"></param>
        /// <param name="getApiUrlFunc">获取服务器地址的方法</param>
        /// <param name="useTrackId"></param>
        /// <param name="retryTiems">重试次数和时间</param> 
        /// <returns></returns>
        public static IServiceCollection UseApiSDK<TApiSDK>(
            this IServiceCollection services,
            Func<IServiceProvider, string> getApiUrlFunc,
            bool useTrackId = true,
            params TimeSpan[] retryTiems)
            where TApiSDK : BaseApiSDK<TApiSDK>
        {
            ApiUrlFuncCollection.Set<TApiSDK>(getApiUrlFunc);
            var fullName = typeof(TApiSDK).FullName;
            if (fullName != null)
            {
                var httpBuilder = services.AddHttpClient(fullName);
                if (useTrackId)
                {
                    httpBuilder.AddHttpMessageHandler<SDKHttpMessageHandlerWithTrackId>();
                }
                else
                {
                    httpBuilder.AddHttpMessageHandler<SDKHttpMessageHandler>();
                }
            }

            services.AddTransient<TApiSDK>();
            return services;
        }
    }
}