﻿using BPA.Component.DTOCommon.Langs;
using Microsoft.AspNetCore.Builder;
using Microsoft.Extensions.DependencyInjection;

namespace BPA.Component.WebApiExtensions.Extensions
{
    /// <summary>
    /// 语言包扩展
    /// </summary>
    public static class LangPackageExtension
    {
        /// <summary>
        /// 注入语言包,单例,必须继承<see cref="BaseLangPackge"/>
        /// </summary>
        /// <typeparam name="TLangPackage"></typeparam>
        /// <param name="services"></param>
        public static IServiceCollection AddLangPackage<TLangPackage>(this IServiceCollection services) where TLangPackage : BaseLangPackge
        {
            services.AddSingleton<TLangPackage>();
            services.AddSingleton<BaseLangPackge>(sp => sp.GetService<TLangPackage>()
                                                        ?? throw new InvalidOperationException("TLangPackage null"));
            return services;
        }

        /// <summary>
        /// 启用语言包
        /// </summary>
        /// <typeparam name="TLangPackage"></typeparam>
        /// <param name="builder"></param>
        public static TLangPackage? UseLangPackage<TLangPackage>(this IApplicationBuilder builder) where TLangPackage : BaseLangPackge
        {
            return builder.ApplicationServices.GetService<TLangPackage>();
        }
    }
}