﻿using System.Diagnostics;
using BPA.Component.DTOCommon.BaseDTOs;
using BPA.Component.DTOCommon.Langs;
using BPA.Component.Extensions;
using BPA.Component.WebApiExtensions.Extensions;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;

namespace BPA.Component.WebApiExtensions.Filters;

/// <summary>
/// BaseResultFilter
/// </summary>
public class BaseResultFilter : Attribute, IActionFilter, IOrderedFilter, IDisposable
{
    private ILogger<BaseResultFilter>? _logger;
    private Stopwatch _stopwatch = new();

    public void OnActionExecuting(ActionExecutingContext context)
    {
        _stopwatch.Restart();
    }

    public void OnActionExecuted(ActionExecutedContext context)
    {
        if (context.Result is not ObjectResult objectResult)
        {
            return;
        }

        switch (objectResult.Value)
        {
            case BaseResult baseResult:
            {
                _stopwatch.Stop();
                baseResult.TrackId = context.HttpContext.TraceIdentifier;
                baseResult.Duration = $"{_stopwatch.ElapsedMilliseconds}ms";
                baseResult.Render(context.HttpContext.Request.GetLanguage());

                if (!baseResult.IsOK && baseResult.Code != ConstLangPackge.LoginExpired.Key)
                {
                    _logger ??= context.HttpContext.RequestServices.GetService<ILogger<BaseResultFilter>>();
                    _logger.LogWarning(
                        "拦截到处理失败:\n" +
                        $"【reqUrl】:\n\t{context.HttpContext.Request.GetRequestUrl()}\n" +
                        $"【reqArgs】:\n\t{context.HttpContext.Request.ReadBodyAsync().Result} \n" +
                        $"【rspArgs】:\n\t{baseResult.ToJson()}");
                }

                break;
            }
        }
    }

    /// <summary>
    /// 排序
    /// </summary>
    public int Order => 0;

    public void Dispose()
    {
        _stopwatch = null;
    }
}