﻿using BPA.Component.DTOCommon.BaseDTOs;
using BPA.Component.DTOCommon.Langs;
using BPA.Component.WebApiExtensions.Extensions;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using Microsoft.Extensions.Logging;

namespace BPA.Component.WebApiExtensions.Filters
{
    /// <summary>
    /// ExceptionFilter, 内部错误，始终保证返回CODE 200
    /// </summary>
    public class ExceptionFilter : Attribute, IExceptionFilter, IOrderedFilter
    {
        /// <summary>
        /// 是否网关
        /// </summary>
        internal static bool IsGeteway = false;

        /// <summary>
        /// 排序
        /// </summary>
        public int Order => 1;

        /// <summary>
        /// logger
        /// </summary>
        private readonly ILogger<ExceptionFilter> _logger;

        /// <summary>
        /// ExceptionFilter
        /// </summary>
        /// <param name="logger"></param>
        public ExceptionFilter(ILogger<ExceptionFilter> logger)
        {
            _logger = logger;
        }

        /// <summary>
        /// 异常处理
        /// </summary>
        /// <param name="context"></param>
        public void OnException(ExceptionContext context)
        {
            BaseResult result;
            var statusCode = 200;
            if (context.Exception is BusinessException businessException)
            {
                _logger.LogWarning(context.Exception,
                    $@"{businessException.Message}\n" +
                    $@"【reqUrl】:\n\t{context.HttpContext.Request.GetRequestUrl()}\n" +
                    $@"【reqArgs】:\n\t{context.HttpContext.Request.ReadBodyAsync().Result}");
                context.ExceptionHandled = true;
                result = BaseResult.Build(businessException.Lang, businessException.MoreZhMsg, businessException.MoreEnMsg);
                statusCode = 200;
            }
            else
            {
                if (IsGeteway)
                {
                    var cookies = string.Join("\n\t", context.HttpContext.Request.Cookies.Select(x => $"{x.Key}={x.Value}"));
                    _logger.LogError(context.Exception,
                        $"未处理异常,\n" +
                        $"【reqUrl】:\n\t{context.HttpContext.Request.GetRequestUrl()} \n " +
                        $"【cookies】:\n\t{cookies} \n" +
                        $"【reqArgs】:\n\t{context.HttpContext.Request.ReadBodyAsync().Result} ");
                }
                else
                {
                    _logger.LogError(context.Exception,
                        $"未处理异常,\n" +
                        $"【reqUrl】:\n\t{context.HttpContext.Request.GetRequestUrl()}\n" +
                        $"【reqArgs】:\n\t{context.HttpContext.Request.ReadBodyAsync().Result} ");
                }

                result = BaseResult.Build(ConstLangPackge.Error, "请稍后重试", "please try again later");
                if (IsGeteway)
                {
                    context.ExceptionHandled = true;
                }
                else
                {
                    statusCode = 500;
                    context.ExceptionHandled = false;
                }
            }

            result.Render(context.HttpContext.Request.GetLanguage());
            result.TrackId = context.HttpContext.TraceIdentifier;
            context.Result = new ObjectResult(result) {StatusCode = statusCode};
        }
    }
}