﻿using BPA.Component.DTOCommon.Validations;
using BPA.Component.WebApiExtensions.Extensions;
using Microsoft.AspNetCore.Mvc.ModelBinding.Metadata;
using Microsoft.AspNetCore.Mvc.ModelBinding.Validation;

namespace BPA.Component.WebApiExtensions.ValidationExtensions
{
    /// <summary>
    /// BPAModelValidator
    /// </summary>
    public class BPAModelValidator : IModelValidator
    {
        /// <summary>
        /// IBPAValidationAttribute
        /// </summary>
        private readonly BPAValidationAttribute _attribute;

        /// <summary>
        /// 是否为应用网关层
        /// </summary>
        private readonly bool isGeteway;

        /// <summary>
        /// BPAModelValidator
        /// </summary>
        public BPAModelValidator(BPAValidationAttribute attribute, bool isGeteway)
        {
            _attribute = attribute;
            this.isGeteway = isGeteway;
        }

        /// <summary>
        /// context
        /// </summary>
        /// <param name="context"></param>
        /// <returns></returns>
        public IEnumerable<ModelValidationResult> Validate(ModelValidationContext context)
        {
            var result = new List<ModelValidationResult>();
            if (isGeteway)
            {
                if (context.ModelMetadata is DefaultModelMetadata modelMetadata)
                {
                    var attributes = modelMetadata.Attributes;
                    if (attributes.PropertyAttributes != null)
                    {
                        if (attributes.PropertyAttributes.Any(attr => attr.GetType() == typeof(BPAExcludeGetewayAttribute)))
                        {
                            return result;
                        }
                    }
                }
            }

            if (_attribute.IsValid(context.Model))
                return result;
            var langType = context.ActionContext.HttpContext.Request.GetLanguage();
            //根据语言类型,获取验证错误信息
            var errorMessage = _attribute.GetErrorMessage(langType);
            result.Add(new ModelValidationResult(string.Empty, errorMessage));
            return result;
        }
    }
}